@@ -1,14 +1,12 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /*
  * brasero
- * Copyright (C) Philippe Rouquier 2007 <bonfire-app@wanadoo.fr>
+ * Copyright (C) Philippe Rouquier 2007-2008 <bonfire-app@wanadoo.fr>
  * 
- * brasero is free software.
- * 
- * You may redistribute it and/or modify it under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option)
- * any later version.
+ *  Brasero is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  * 
  * brasero is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -37,6 +35,8 @@
 #include "brasero-file-monitor.h"
 #include "burn-debug.h"
 
+#include "brasero-file-node.h"
+
 typedef struct _BraseroFileMonitorPrivate BraseroFileMonitorPrivate;
 struct _BraseroFileMonitorPrivate
 {
@@ -109,6 +109,8 @@ brasero_file_monitor_moved_to_event (BraseroFileMonitor *self,
 	priv = BRASERO_FILE_MONITOR_PRIVATE (self);
 	klass = BRASERO_FILE_MONITOR_GET_CLASS (self);
 
+	BRASERO_BURN_LOG ("File Monitoring (move to for %s)", name);
+
 	if (!cookie) {
 		if (klass->file_added)
 			klass->file_added (self, callback_data, name);
@@ -181,6 +183,8 @@ brasero_file_monitor_move_timeout_cb (BraseroFileMonitor *self)
 	data = priv->moved_list->data;
 	priv->moved_list = g_slist_remove (priv->moved_list, data);
 
+	BRASERO_BURN_LOG ("File Monitoring (move timeout for %s)", data->name);
+
 	if (klass->file_removed)
 		klass->file_removed (self,
 				     data->type,
@@ -205,6 +209,8 @@ brasero_file_monitor_moved_from_event (BraseroFileMonitor *self,
 
 	priv = BRASERO_FILE_MONITOR_PRIVATE (self);
 
+	BRASERO_BURN_LOG ("File Monitoring (moved from event for %s)", name);
+
 	if (!cookie) {
 		BraseroFileMonitorClass *klass;
 
@@ -250,20 +256,25 @@ brasero_file_monitor_directory_event (BraseroFileMonitor *self,
 	 * This is done to avoid treating events twice.
 	 * IN_DELETE_SELF or IN_MOVE_SELF are therefore not possible here. */
 	if (event->mask & IN_ATTRIB) {
+		BRASERO_BURN_LOG ("File Monitoring (attributes changed for %s)", name);
 		if (klass->file_modified)
 			klass->file_modified (self, callback_data, name);
 	}
 	else if (event->mask & IN_MODIFY) {
+		BRASERO_BURN_LOG ("File Monitoring (modified for %s)", name);
 		if (klass->file_modified)
 			klass->file_modified (self, callback_data, name);
 	}
 	else if (event->mask & IN_MOVED_FROM) {
+		BRASERO_BURN_LOG ("File Monitoring (moved from for %s)", name);
 		brasero_file_monitor_moved_from_event (self, type, callback_data, name, event->cookie);
 	}
 	else if (event->mask & IN_MOVED_TO) {
+		BRASERO_BURN_LOG ("File Monitoring (moved to for %s)", name);
 		brasero_file_monitor_moved_to_event (self, callback_data, name, event->cookie);
 	}
 	else if (event->mask & (IN_DELETE|IN_UNMOUNT)) {
+		BRASERO_BURN_LOG ("File Monitoring (delete/unmount for %s)", name);
 		if (klass->file_removed)
 			klass->file_removed (self,
 					     type,
@@ -271,6 +282,7 @@ brasero_file_monitor_directory_event (BraseroFileMonitor *self,
 					     name);
 	}
 	else if (event->mask & IN_CREATE) {
+		BRASERO_BURN_LOG ("File Monitoring (create for %s)", name);
 		if (klass->file_added)
 			klass->file_added (self, callback_data, name);
 	}
@@ -520,8 +532,7 @@ brasero_file_monitor_inotify_monitor_cb (GIOChannel *channel,
 		callback_data = g_hash_table_lookup (priv->files, GINT_TO_POINTER (event.wd));
 		if (!callback_data) {
 			/* Retry with children */
-			callback_data = g_hash_table_lookup (priv->directories,
-							     GINT_TO_POINTER (event.wd));
+			callback_data = g_hash_table_lookup (priv->directories, GINT_TO_POINTER (event.wd));
 			if (name && callback_data) {
 				/* For directories we don't take heed of the SELF events.
 				 * All events are treated through the parent directory
@@ -562,6 +573,7 @@ brasero_file_monitor_start_monitoring_real (BraseroFileMonitor *self,
 					    const gchar *uri)
 {
 	BraseroFileMonitorPrivate *priv;
+	gchar *unescaped_uri;
 	gchar *path;
 	gint dev_fd;
 	uint32_t mask;
@@ -569,7 +581,9 @@ brasero_file_monitor_start_monitoring_real (BraseroFileMonitor *self,
 
 	priv = BRASERO_FILE_MONITOR_PRIVATE (self);
 
-	path = g_filename_from_uri (uri, NULL, NULL);
+	unescaped_uri = g_uri_unescape_string (uri, NULL);
+	path = g_filename_from_uri (unescaped_uri, NULL, NULL);
+	g_free (unescaped_uri);
 
 	dev_fd = g_io_channel_unix_get_fd (priv->notify);
 	mask = IN_MODIFY |
@@ -581,11 +595,12 @@ brasero_file_monitor_start_monitoring_real (BraseroFileMonitor *self,
 	       IN_DELETE_SELF |
 	       IN_MOVE_SELF;
 
+	/* NOTE: always return the same wd when we ask for the same file */
 	wd = inotify_add_watch (dev_fd, path, mask);
 	if (wd == -1) {
 		BRASERO_BURN_LOG ("ERROR creating watch for local file %s : %s\n",
 				  path,
-				  strerror (errno));
+				  g_strerror (errno));
 		g_free (path);
 		return 0;
 	}
@@ -628,6 +643,7 @@ brasero_file_monitor_single_file (BraseroFileMonitor *self,
 	data->callback_data = callback_data;
 	BRASERO_GET_BASENAME_FOR_DISPLAY (uri, data->name);
 
+	/* inotify always return the same wd for the same file */
 	list = g_hash_table_lookup (priv->files, GINT_TO_POINTER (wd));
 	list = g_slist_prepend (list, data);
 	g_hash_table_insert (priv->files,
@@ -658,6 +674,7 @@ brasero_file_monitor_directory_contents (BraseroFileMonitor *self,
 	 * parent directory. We give them the same handle as their parent
 	 * directory to find it more easily and mark it as being watched */
 	wd = brasero_file_monitor_start_monitoring_real (self, uri);
+
 	if (!wd)
 		return FALSE;
 
@@ -679,7 +696,7 @@ brasero_file_monitor_foreach_cancel_file_cb (gpointer key,
 		BraseroInotifyFileData *file_data = NULL;
 
 		file_data = iter->data;
-		if (data->func (data->callback_data, file_data->callback_data)) {
+		if (data->func (file_data->callback_data, data->callback_data)) {
 			BraseroFileMonitorSearchResult *result;
 
 			result = g_new0 (BraseroFileMonitorSearchResult, 1);
@@ -713,6 +730,7 @@ brasero_file_monitor_foreach_cancel (BraseroFileMonitor *self,
 				     gpointer callback_data)
 {
 	GSList *iter;
+	GSList *next;
 	BraseroFileMonitorPrivate *priv;
 	BraseroFileMonitorCancelForeach data;
 
@@ -751,6 +769,20 @@ brasero_file_monitor_foreach_cancel (BraseroFileMonitor *self,
 	g_hash_table_foreach_remove (priv->directories,
 				     brasero_file_monitor_foreach_cancel_directory_cb,
 				     &data);
+
+	/* Finally get rid of moved that data in moved list */
+	for (iter = priv->moved_list; iter; iter = next) {
+		BraseroInotifyMovedData *data;
+
+		data = iter->data;
+		next = iter->next;
+		if (func (data->callback_data, callback_data)) {
+			priv->moved_list = g_slist_remove (priv->moved_list, data);
+			g_source_remove (data->id);
+			g_free (data->name);
+			g_free (data);
+		}
+	}
 }
 
 static gboolean
@@ -820,7 +852,7 @@ brasero_file_monitor_init (BraseroFileMonitor *object)
 		g_io_channel_unref (priv->notify);
 	}
 	else
-		g_warning ("Failed to open inotify: %s\n", strerror (errno));
+		g_warning ("Failed to open inotify: %s\n", g_strerror (errno));
 }
 
 static void
