@@ -1,14 +1,12 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /*
  * brasero
- * Copyright (C) Philippe Rouquier 2008 <bonfire-app@wanadoo.fr>
+ * Copyright (C) Philippe Rouquier 2005-2008 <bonfire-app@wanadoo.fr>
  * 
- * brasero is free software.
- * 
- * You may redistribute it and/or modify it under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option)
- * any later version.
+ *  Brasero is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  * 
  * brasero is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -97,6 +95,7 @@ brasero_file_filtered_gconf_notify_cb (GConfClient *client,
 static void
 brasero_filter_option_init (BraseroFilterOption *object)
 {
+	gchar *string;
 	gboolean active;
 	GtkWidget *frame;
 	GError *error = NULL;
@@ -133,7 +132,7 @@ brasero_filter_option_init (BraseroFilterOption *object)
 	active = gconf_client_get_bool (priv->client,
 					BRASERO_FILTER_BROKEN_SYM_KEY,
 					NULL);
-	
+
 	button_sym = gtk_check_button_new_with_mnemonic (_("Filter _broken symlinks"));
 	gtk_widget_show (button_sym);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button_sym), active);
@@ -152,10 +151,13 @@ brasero_filter_option_init (BraseroFilterOption *object)
 		error = NULL;
 	}
 
-	frame = brasero_utils_pack_properties (_("<b>Filtering options</b>"),
+	string = g_strdup_printf ("<b>%s</b>", _("Filtering options"));
+	frame = brasero_utils_pack_properties (string,
 					       button_sym,
 					       button_hidden,
 					       NULL);
+	g_free (string);
+
 	gtk_box_pack_start (GTK_BOX (object),
 			    frame,
 			    FALSE,
