@@ -7,19 +7,21 @@
 ****************************************************************************/
 
 /*
- *  This program is free software; you can redistribute it and/or modify
+ *  Brasero is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Brasero is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.
  */
 
 
@@ -812,7 +814,7 @@ static void
 brasero_playlist_dialog_error (BraseroPlaylist *playlist, const gchar *uri)
 {
 	gchar *name;
-	GtkWidget *dialog;
+	gchar *primary;
 	GtkWidget *toplevel;
 
 	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (playlist));
@@ -822,22 +824,14 @@ brasero_playlist_dialog_error (BraseroPlaylist *playlist, const gchar *uri)
 	}
 
 	BRASERO_GET_BASENAME_FOR_DISPLAY (uri, name);
-	dialog = gtk_message_dialog_new (GTK_WINDOW (toplevel),
-					 GTK_DIALOG_DESTROY_WITH_PARENT|
-					 GTK_DIALOG_MODAL,
-					 GTK_MESSAGE_ERROR,
-					 GTK_BUTTONS_CLOSE,
-					 _("Error parsing playlist \"%s\":"),
-					 name);
+
+	primary = g_strdup_printf (_("Error parsing playlist \"%s\":"), name);
+	brasero_utils_message_dialog (GTK_WIDGET (toplevel),
+				      primary,
+				      _("an unknown error occured."),
+				      GTK_MESSAGE_ERROR);
+	g_free (primary);
 	g_free (name);
-
-	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-						  _("an unknown error occured."));
-
-	gtk_window_set_title (GTK_WINDOW (dialog), _("Playlist Loading Error"));
-
-	gtk_dialog_run (GTK_DIALOG (dialog));
-	gtk_widget_destroy (dialog);
 }
 
 static void
@@ -977,7 +971,7 @@ brasero_playlist_insert (BraseroPlaylist *playlist, const gchar *uri)
 	gtk_tree_store_set (GTK_TREE_STORE (model), &parent,
 			    BRASERO_PLAYLIST_DISPLAY_COL, name,
 			    BRASERO_PLAYLIST_URI_COL, uri,
-			    BRASERO_PLAYLIST_NB_SONGS_COL, _("Loading ..."),
+			    BRASERO_PLAYLIST_NB_SONGS_COL, _("(loading ...)"),
 			    -1);
 	g_free (name);
 
