@@ -7,20 +7,21 @@
  * Based on gedit code (gedit/gedit-plugin-manager.c) by: 
  * 	- Paolo Maggi <paolo@gnome.org>
  *
- * This program is free software; you can redistribute it and/or modify
+ * Brasero is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * Brasero is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, 
- * Boston, MA 02111-1307, USA. 
+ * along with this program; if not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -214,7 +215,7 @@ plugin_manager_ui_view_info_cell_cb (GtkTreeViewColumn *tree_column,
 		return;
 
 	if (brasero_plugin_get_error (plugin))
-		text = g_markup_printf_escaped (_("<b>%s</b>\n%s\n<i>%s</i>"),
+		text = g_markup_printf_escaped ("<b>%s</b>\n%s\n<i>%s</i>",
 						brasero_plugin_get_name (plugin),
 						brasero_plugin_get_description (plugin),
 						brasero_plugin_get_error (plugin));
@@ -354,9 +355,9 @@ plugin_manager_ui_populate_lists (BraseroPluginManagerUI *pm)
 	for (; plugins; plugins = plugins->next) {
 		plugin = plugins->data;
 
-		if (brasero_burn_caps_plugin_can_burn (caps, plugin) == BRASERO_BURN_OK
-		||  brasero_burn_caps_plugin_can_convert (caps, plugin) == BRASERO_BURN_OK
-		||  brasero_burn_caps_plugin_can_image (caps, plugin) == BRASERO_BURN_OK
+		if (brasero_plugin_can_burn (plugin) == BRASERO_BURN_OK
+		||  brasero_plugin_can_convert (plugin) == BRASERO_BURN_OK
+		||  brasero_plugin_can_image (plugin) == BRASERO_BURN_OK
 		||  brasero_plugin_get_gtype (plugin) == G_TYPE_NONE)
 			continue;
 
