@@ -7,19 +7,21 @@
  ***************************************************************************/
 
 /*
- *  This program is free software; you can redistribute it and/or modify
+ *  Brasero is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Brasero is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.
  */
 
 #include <string.h>
@@ -43,6 +45,7 @@
 #include <gtk/gtkuimanager.h>
 #include <gtk/gtkscrolledwindow.h>
 
+#include "brasero-app.h"
 #include "brasero-utils.h"
 #include "brasero-project.h"
 #include "brasero-layout.h"
@@ -55,6 +58,7 @@
 #include "brasero-disc-copy-dialog.h"
 #include "brasero-io.h"
 #include "burn-caps.h"
+#include "burn-medium-monitor.h"
 
 #ifdef BUILD_SEARCH
 #include "brasero-search-beagle.h"
@@ -86,6 +90,8 @@ brasero_project_manager_new_audio_prj_cb (GtkAction *action, BraseroProjectManag
 static void
 brasero_project_manager_new_data_prj_cb (GtkAction *action, BraseroProjectManager *manager);
 static void
+brasero_project_manager_new_video_prj_cb (GtkAction *action, BraseroProjectManager *manager);
+static void
 brasero_project_manager_new_copy_prj_cb (GtkAction *action, BraseroProjectManager *manager);
 static void
 brasero_project_manager_new_iso_prj_cb (GtkAction *action, BraseroProjectManager *manager);
@@ -115,6 +121,8 @@ static GtkActionEntry entries [] = {
 	 N_("Create a new audio project"), G_CALLBACK (brasero_project_manager_new_audio_prj_cb)},
 	{"NewData", "media-optical-data-new", N_("New _Data Project"), NULL,
 	 N_("Create a new data project"), G_CALLBACK (brasero_project_manager_new_data_prj_cb)},
+	{"NewVideo", "media-optical-video-new", N_("New _Video Project"), NULL,
+	 N_("Create a new video project"), G_CALLBACK (brasero_project_manager_new_video_prj_cb)},
 	{"NewCopy", "media-optical-copy", N_("Copy _Disc..."), NULL,
 	 N_("Copy a disc"), G_CALLBACK (brasero_project_manager_new_copy_prj_cb)},
 	{"NewIso", "iso-image-burn", N_("_Burn Image..."), NULL,
@@ -132,6 +140,7 @@ static const char *description = {
 				"<menu action='New'>"
 					"<menuitem action='NewAudio'/>"
 					"<menuitem action='NewData'/>"
+					"<menuitem action='NewVideo'/>"
 					"<menuitem action='NewCopy'/>"	
 					"<menuitem action='NewIso'/>"	
 				"</menu>"
@@ -166,11 +175,15 @@ struct BraseroProjectManagerPrivate {
 
 	GtkWidget *project;
 	GtkWidget *layout;
-	GtkWidget *status;
+
+	gchar **selected;
+	guint preview_id;
 
 	guint status_ctx;
 
 	GtkActionGroup *action_group;
+
+	guint oneshot:1;
 };
 
 #define BRASERO_PROJECT_MANAGER_CONNECT_CHANGED(manager, container)		\
@@ -230,10 +243,18 @@ brasero_project_manager_set_statusbar (BraseroProjectManager *manager,
 				       gint files_num)
 {
 	gchar *status_string = NULL;
+	GtkWidget *toplevel;
+	GtkWidget *status;
 	gint valid_num;
 
-	gtk_statusbar_pop (GTK_STATUSBAR (manager->priv->status),
-			   manager->priv->status_ctx);
+	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (manager));
+	status = brasero_app_get_statusbar1 (BRASERO_APP (toplevel));
+
+	if (!manager->priv->status_ctx)
+		manager->priv->status_ctx = gtk_statusbar_get_context_id (GTK_STATUSBAR (status),
+									  "size_info");
+
+	gtk_statusbar_pop (GTK_STATUSBAR (status), manager->priv->status_ctx);
 
 	valid_num = files_num - invalid_num;
 	if (!invalid_num && valid_num) {
@@ -286,8 +307,7 @@ brasero_project_manager_set_statusbar (BraseroProjectManager *manager,
 	else
 		status_string = g_strdup (_("No file selected"));
 
-	gtk_statusbar_push (GTK_STATUSBAR (manager->priv->status),
-			    manager->priv->status_ctx,
+	gtk_statusbar_push (GTK_STATUSBAR (status), manager->priv->status_ctx,
 			    status_string);
 	g_free (status_string);
 }
@@ -330,29 +350,13 @@ brasero_project_manager_size_preview_progress (GObject *object,
 					       files_num);
 }
 
-void
-brasero_project_manager_selected_uris_changed (BraseroURIContainer *container,
-					       BraseroProjectManager *manager)
+static gboolean
+brasero_project_manager_selected_uris_preview (gpointer data)
 {
+	BraseroProjectManager *manager = BRASERO_PROJECT_MANAGER (data);
 	BraseroIOFlags flags;
-    	gchar **uris, **iter;
-    	GSList *list = NULL;
-
-	/* if we are in the middle of an unfinished size seek then
-	 * cancel it and re-initialize */
-	if (manager->priv->io)
-		brasero_io_cancel_by_base (manager->priv->io,
-					   manager->priv->size_preview);
-
-	uris = brasero_uri_container_get_selected_uris (container);
-	if (!uris) {
-		gtk_statusbar_pop (GTK_STATUSBAR (manager->priv->status),
-				   manager->priv->status_ctx);
-		gtk_statusbar_push (GTK_STATUSBAR (manager->priv->status),
-				    manager->priv->status_ctx,
-				    _("No file selected"));
-		return;
-	}
+	GSList *list = NULL;
+	gchar **iter;
 
 	if (!manager->priv->io)
 		manager->priv->io = brasero_io_get_default ();
@@ -363,7 +367,7 @@ brasero_project_manager_selected_uris_changed (BraseroURIContainer *container,
 								   NULL,
 								   brasero_project_manager_size_preview_progress);
     
-	for (iter = uris; iter && *iter; iter ++)
+	for (iter = manager->priv->selected; iter && *iter; iter ++)
 		list = g_slist_prepend (list, *iter);
 
 	flags = BRASERO_IO_INFO_RECURSIVE|BRASERO_IO_INFO_IDLE;
@@ -377,15 +381,126 @@ brasero_project_manager_selected_uris_changed (BraseroURIContainer *container,
 				   NULL);
 			       
 	g_slist_free (list);
-	g_strfreev (uris);
+	manager->priv->preview_id = 0;
+	return FALSE;
 }
+
 void
-brasero_project_manager_set_status (BraseroProjectManager *manager,
-				    GtkWidget *status)
+brasero_project_manager_selected_uris_changed (BraseroURIContainer *container,
+					       BraseroProjectManager *manager)
 {
-	manager->priv->status = status;
-	manager->priv->status_ctx = gtk_statusbar_get_context_id (GTK_STATUSBAR (status),
-								  "size_info");
+	gchar **uris;
+
+	/* Before cancelling everything, check that the size really changed
+	 * like in the case of double clicking or if the user selected one
+	 * file and double clicked on it afterwards.
+	 * NOTE: the following expects each URI to be unique. */
+	uris = brasero_uri_container_get_selected_uris (container);
+	if (uris) {
+		gchar **iter;
+		guint num = 0;
+		gboolean found = FALSE;
+
+		for (iter = manager->priv->selected; iter && *iter; iter ++) {
+			gchar **uri;
+
+			found = FALSE;
+			for (uri = uris; uri && *uri; uri ++) {
+				if (!strcmp (*uri, *iter)) {
+					found = TRUE;
+					break;
+				}
+			}
+
+			if (!found)
+				break;
+
+			num ++;
+		}
+
+		if (found) {
+			guint num_new = 0;
+
+			for (iter = uris; iter && *iter; iter ++)
+				num_new ++;
+
+			if (num_new == num) {
+				/* same uris no need to update anything. */
+				g_strfreev (uris);
+				return;
+			}
+		}
+	}
+
+	/* if we are in the middle of an unfinished size seek then
+	 * cancel it and re-initialize */
+	if (manager->priv->io)
+		brasero_io_cancel_by_base (manager->priv->io,
+					   manager->priv->size_preview);
+
+	if (manager->priv->selected) {
+		g_strfreev (manager->priv->selected);
+		manager->priv->selected = NULL;
+	}
+
+	if (manager->priv->preview_id) {
+		g_source_remove (manager->priv->preview_id);
+		manager->priv->preview_id = 0;
+	}
+
+	manager->priv->selected = uris;
+	if (!manager->priv->selected) {
+		GtkWidget *toplevel;
+ 		GtkWidget *status;
+ 
+ 		toplevel = gtk_widget_get_toplevel (GTK_WIDGET (manager));
+ 		status = brasero_app_get_statusbar1 (BRASERO_APP (toplevel));
+ 
+ 		if (!manager->priv->status_ctx)
+ 			manager->priv->status_ctx = gtk_statusbar_get_context_id (GTK_STATUSBAR (status),
+ 										  "size_info");
+ 
+ 		gtk_statusbar_pop (GTK_STATUSBAR (status), manager->priv->status_ctx);
+ 		gtk_statusbar_push (GTK_STATUSBAR (status),
+  				    manager->priv->status_ctx,
+  				    _("No file selected"));
+		return;
+	}
+
+	manager->priv->preview_id = g_timeout_add (500,
+						   brasero_project_manager_selected_uris_preview,
+						   manager);
+}
+
+void
+brasero_project_manager_sidepane_changed (BraseroLayout *layout,
+					  gboolean visible,
+					  BraseroProjectManager *manager)
+{
+	if (!visible) {
+		GtkWidget *toplevel;
+ 		GtkWidget *status;
+ 
+		/* If sidepane is disabled, remove any text about selection */
+		if (manager->priv->io)
+			brasero_io_cancel_by_base (manager->priv->io,
+						   manager->priv->size_preview);
+
+ 		toplevel = gtk_widget_get_toplevel (GTK_WIDGET (manager));
+ 		status = brasero_app_get_statusbar1 (BRASERO_APP (toplevel));
+
+ 		gtk_statusbar_pop (GTK_STATUSBAR (status), manager->priv->status_ctx);
+
+		if (manager->priv->selected) {
+			g_strfreev (manager->priv->selected);
+			manager->priv->selected = NULL;
+		}
+
+		if (manager->priv->preview_id) {
+			g_source_remove (manager->priv->preview_id);
+			manager->priv->preview_id = 0;
+		}
+	}
 }
 
 void
@@ -463,21 +578,36 @@ brasero_project_manager_burn_iso_dialog (BraseroProjectManager *manager,
 	result = gtk_dialog_run (GTK_DIALOG (dialog));
 	if (result != GTK_RESPONSE_OK) {
 		gtk_widget_destroy (dialog);
+
+		/* Here we may have to close brasero altogether */
+		if (manager->priv->oneshot) {
+			GtkWidget *toplevel;
+
+			toplevel = gtk_widget_get_toplevel (GTK_WIDGET (manager));
+			gtk_widget_destroy (toplevel);
+		}
+
 		return;
 	}
 
-	session = brasero_image_option_dialog_get_session (BRASERO_IMAGE_OPTION_DIALOG (dialog));
+	session = brasero_burn_options_get_session (BRASERO_BURN_OPTIONS (dialog));
 	gtk_widget_destroy (dialog);
 
-	if (!session)
-		return;
-
 	brasero_project_manager_burn (manager, session);
 	g_object_unref (session);
+
+	/* Here we may have to close brasero altogether */
+	if (manager->priv->oneshot) {
+		GtkWidget *toplevel;
+
+		toplevel = gtk_widget_get_toplevel (GTK_WIDGET (manager));
+		gtk_widget_destroy (toplevel);
+	}
 }
 
 static void
-brasero_project_manager_burn_disc (BraseroProjectManager *manager)
+brasero_project_manager_copy_disc (BraseroProjectManager *manager,
+				   const gchar *device)
 {
 	BraseroBurnSession *session;
 	GtkResponseType result;
@@ -492,17 +622,47 @@ brasero_project_manager_burn_disc (BraseroProjectManager *manager)
 	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER_ON_PARENT);
 	gtk_widget_show (dialog);
 
+	/* if a device is specified then get the corresponding medium */
+	if (device) {
+		BraseroDrive *drive;
+		BraseroMediumMonitor *monitor;
+
+		monitor = brasero_medium_monitor_get_default ();
+		drive = brasero_medium_monitor_get_drive (monitor, device);
+		g_object_unref (monitor);
+
+		brasero_disc_copy_dialog_set_drive (BRASERO_DISC_COPY_DIALOG (dialog), drive);
+		g_object_unref (drive);
+	}
+
 	result = gtk_dialog_run (GTK_DIALOG (dialog));
 	if (result != GTK_RESPONSE_OK) {
 		gtk_widget_destroy (dialog);
+
+		/* Here we may have to close brasero altogether */
+		if (manager->priv->oneshot) {
+			GtkWidget *toplevel;
+
+			toplevel = gtk_widget_get_toplevel (GTK_WIDGET (manager));
+			gtk_widget_destroy (toplevel);
+		}
+
 		return;
 	}
 
-	session = brasero_disc_copy_dialog_get_session (BRASERO_DISC_COPY_DIALOG (dialog));
+	session = brasero_burn_options_get_session (BRASERO_BURN_OPTIONS (dialog));
 	gtk_widget_destroy (dialog);
 
 	brasero_project_manager_burn (manager, session);
 	g_object_unref (session);
+
+	/* Here we may have to close brasero altogether */
+	if (manager->priv->oneshot) {
+		GtkWidget *toplevel;
+
+		toplevel = gtk_widget_get_toplevel (GTK_WIDGET (manager));
+		gtk_widget_destroy (toplevel);
+	}
 }
 
 static void
@@ -515,15 +675,28 @@ brasero_project_manager_switch (BraseroProjectManager *manager,
 	GtkWidget *toplevel;
 	GtkAction *action;
 
-	if ((manager->priv->type == BRASERO_PROJECT_TYPE_AUDIO
-	||   manager->priv->type == BRASERO_PROJECT_TYPE_DATA)
-	&&  !brasero_project_confirm_switch (BRASERO_PROJECT (manager->priv->project)))
-		return;
-
-	gtk_statusbar_pop (GTK_STATUSBAR (manager->priv->status),
-			   manager->priv->status_ctx);
-
 	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (manager));
+
+	if (manager->priv->type == BRASERO_PROJECT_TYPE_AUDIO
+	||  manager->priv->type == BRASERO_PROJECT_TYPE_DATA
+	||  manager->priv->type == BRASERO_PROJECT_TYPE_VIDEO) {
+		if (!brasero_project_confirm_switch (BRASERO_PROJECT (manager->priv->project)))
+			return;
+
+		if (manager->priv->oneshot) {
+			/* Here we may have to close brasero altogether */
+			gtk_widget_destroy (toplevel);
+			return;
+		}
+	}
+
+	if (manager->priv->status_ctx) {
+		GtkWidget *status;
+
+		status = brasero_app_get_statusbar1 (BRASERO_APP (toplevel));
+		gtk_statusbar_pop (GTK_STATUSBAR (status), manager->priv->status_ctx);
+	}
+
 	action = gtk_action_group_get_action (manager->priv->action_group, "NewChoose");
 
 	manager->priv->type = type;
@@ -566,6 +739,20 @@ brasero_project_manager_switch (BraseroProjectManager *manager,
 		if (toplevel)
 			gtk_window_set_title (GTK_WINDOW (toplevel), _("Brasero - New Data Disc Project"));
 	}
+	else if (type == BRASERO_PROJECT_TYPE_VIDEO) {
+		brasero_layout_load (BRASERO_LAYOUT (manager->priv->layout), BRASERO_LAYOUT_VIDEO);
+
+		gtk_notebook_set_current_page (GTK_NOTEBOOK (manager), 1);
+		gtk_action_set_sensitive (action, TRUE);
+
+		if (reset) {
+			/* tell the BraseroProject object that we want a data selection */
+			brasero_project_set_video (BRASERO_PROJECT (manager->priv->project), uris);
+		}
+
+		if (toplevel)
+			gtk_window_set_title (GTK_WINDOW (toplevel), _("Brasero - New Video disc project"));
+	}
 	else if (type == BRASERO_PROJECT_TYPE_ISO) {
 		brasero_layout_load (BRASERO_LAYOUT (manager->priv->layout), BRASERO_LAYOUT_NONE);
 		brasero_project_set_none (BRASERO_PROJECT (manager->priv->project));
@@ -587,7 +774,7 @@ brasero_project_manager_switch (BraseroProjectManager *manager,
 		if (toplevel)
 			gtk_window_set_title (GTK_WINDOW (toplevel), _("Brasero - Disc Copy"));
 
-		brasero_project_manager_burn_disc (manager);
+		brasero_project_manager_copy_disc (manager, uri);
 	}
 }
 
@@ -596,41 +783,55 @@ brasero_project_manager_type_changed_cb (BraseroProjectTypeChooser *chooser,
 					 BraseroProjectType type,
 					 BraseroProjectManager *manager)
 {
+	manager->priv->oneshot = FALSE;
 	brasero_project_manager_switch (manager, type, NULL, NULL, TRUE);
 }
 
 static void
 brasero_project_manager_new_empty_prj_cb (GtkAction *action, BraseroProjectManager *manager)
 {
+	manager->priv->oneshot = FALSE;
 	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_INVALID, NULL, NULL, TRUE);
 }
 
 static void
 brasero_project_manager_new_audio_prj_cb (GtkAction *action, BraseroProjectManager *manager)
 {
+	manager->priv->oneshot = FALSE;
 	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_AUDIO, NULL, NULL, TRUE);
 }
 
 static void
 brasero_project_manager_new_data_prj_cb (GtkAction *action, BraseroProjectManager *manager)
 {
+	manager->priv->oneshot = FALSE;
 	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_DATA, NULL, NULL, TRUE);
 }
 
+static void
+brasero_project_manager_new_video_prj_cb (GtkAction *action, BraseroProjectManager *manager)
+{
+	manager->priv->oneshot = FALSE;
+	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_VIDEO, NULL, NULL, TRUE);
+}
+
 static void
 brasero_project_manager_new_copy_prj_cb (GtkAction *action, BraseroProjectManager *manager)
 {
+	manager->priv->oneshot = FALSE;
 	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_COPY, NULL, NULL, TRUE);
 }
 
 static void
 brasero_project_manager_new_iso_prj_cb (GtkAction *action, BraseroProjectManager *manager)
 {
+	manager->priv->oneshot = FALSE;
 	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_ISO, NULL, NULL, TRUE);
 }
 
 void
-brasero_project_manager_audio (BraseroProjectManager *manager, GSList *uris)
+brasero_project_manager_audio (BraseroProjectManager *manager,
+			       GSList *uris)
 {
 	brasero_project_manager_switch (manager,
 					BRASERO_PROJECT_TYPE_AUDIO,
@@ -640,25 +841,66 @@ brasero_project_manager_audio (BraseroProjectManager *manager, GSList *uris)
 }
 
 void
-brasero_project_manager_data (BraseroProjectManager *manager, GSList *uris)
+brasero_project_manager_data (BraseroProjectManager *manager,
+			      GSList *uris)
+{
+	gchar *burn_URI = NULL;
+
+	/* always add the contents of burn:/// URI if list is empty */
+	if (!uris) {
+		burn_URI = g_strdup ("burn:///");
+		uris = g_slist_prepend (NULL, burn_URI);
+	}
+
+	if (manager->priv->oneshot) {
+		brasero_project_set_data (BRASERO_PROJECT (manager->priv->project),
+					  uris);
+		brasero_project_burn (BRASERO_PROJECT (manager->priv->project));
+	}
+	else
+		brasero_project_manager_switch (manager,
+						BRASERO_PROJECT_TYPE_DATA,
+						uris,
+						NULL,
+						TRUE);
+	
+	if (burn_URI) {
+		g_slist_free (uris);
+		g_free (burn_URI);
+	}
+}
+
+void
+brasero_project_manager_video (BraseroProjectManager *manager,
+			       GSList *uris)
 {
 	brasero_project_manager_switch (manager,
-					BRASERO_PROJECT_TYPE_DATA,
+					BRASERO_PROJECT_TYPE_VIDEO,
 					uris,
 					NULL,
 					TRUE);
 }
 
 void
-brasero_project_manager_copy (BraseroProjectManager *manager)
+brasero_project_manager_copy (BraseroProjectManager *manager,
+			      const gchar *device)
 {
-	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_COPY, NULL, NULL, TRUE);
+	brasero_project_manager_switch (manager,
+					BRASERO_PROJECT_TYPE_COPY,
+					NULL,
+					device,
+					TRUE);
 }
 
 void
-brasero_project_manager_iso (BraseroProjectManager *manager, const gchar *uri)
+brasero_project_manager_iso (BraseroProjectManager *manager,
+			     const gchar *uri)
 {
-	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_ISO, NULL, uri, TRUE);
+	brasero_project_manager_switch (manager,
+					BRASERO_PROJECT_TYPE_ISO,
+					NULL,
+					uri,
+					TRUE);
 }
 
 BraseroProjectType
@@ -668,12 +910,11 @@ brasero_project_manager_open_project (BraseroProjectManager *manager,
 	BraseroProjectType type;
 	GtkAction *action;
 
-    	gtk_widget_show (manager->priv->layout);
+	gtk_widget_show (manager->priv->layout);
 	gtk_notebook_set_current_page (GTK_NOTEBOOK (manager), 1);
 	type = brasero_project_open_project (BRASERO_PROJECT (manager->priv->project), uri);
 
 	manager->priv->type = type;
-
     	if (type == BRASERO_PROJECT_TYPE_INVALID) {
 		brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_INVALID, NULL, NULL, TRUE);
 		return BRASERO_PROJECT_TYPE_INVALID;
@@ -723,7 +964,10 @@ brasero_project_manager_open_by_mime (BraseroProjectManager *manager,
 				      const gchar *uri,
 				      const gchar *mime)
 {
-	if (!strcmp (mime, "application/x-brasero"))
+	/* When our files/description of x-brasero mime type is not properly 
+	 * installed, it's returned as application/xml, so check that too. */
+	if (!strcmp (mime, "application/x-brasero")
+	||  !strcmp (mime, "application/xml"))
 		return brasero_project_manager_open_project (manager, uri);
 
 #ifdef BUILD_PLAYLIST
@@ -750,39 +994,83 @@ BraseroProjectType
 brasero_project_manager_open_uri (BraseroProjectManager *manager,
 				  const gchar *uri_arg)
 {
-  	GtkWidget *dialog;
-	GtkWidget *window;
 	gchar *uri;
 	GFile *file;
 	GFileInfo *info;
-	const gchar *mime;
 	BraseroProjectType type;
 
 	/* FIXME: make that asynchronous */
+	/* NOTE: don't follow symlink because we want to identify them */
 	file = g_file_new_for_commandline_arg (uri_arg);
 	info = g_file_query_info (file,
-				  G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
-				  G_FILE_QUERY_INFO_NONE,
+				  G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE ","
+				  G_FILE_ATTRIBUTE_STANDARD_IS_SYMLINK ","
+				  G_FILE_ATTRIBUTE_STANDARD_SYMLINK_TARGET,
+				  G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
 				  NULL,
 				  NULL);
 
-	uri = g_file_get_uri (file);
-	mime = g_file_info_get_content_type (info);
+	/* if that's a symlink, redo it on its target to get the real mime type
+	 * that usually also depends on the extension of the target:
+	 * ex: an iso file with the extension .iso will be seen as octet-stream
+	 * if the symlink hasn't got any extention at all */
+	while (g_file_info_get_is_symlink (info)) {
+		const gchar *target;
+		GFileInfo *tmp_info;
+		GFile *tmp_file;
+		GError *error = NULL;
 
+		target = g_file_info_get_symlink_target (info);
+		if (!g_path_is_absolute (target)) {
+			gchar *parent;
+			gchar *tmp;
+
+			tmp = g_file_get_path (file);
+			parent = g_path_get_dirname (tmp);
+			g_free (tmp);
+
+			target = g_build_filename (parent, target, NULL);
+			g_free (parent);
+		}
+
+		tmp_file = g_file_new_for_commandline_arg (target);
+		tmp_info = g_file_query_info (tmp_file,
+					      G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE ","
+					      G_FILE_ATTRIBUTE_STANDARD_IS_SYMLINK ","
+					      G_FILE_ATTRIBUTE_STANDARD_SYMLINK_TARGET,
+					      G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
+					      NULL,
+					      &error);
+		if (!tmp_info) {
+			g_object_unref (tmp_file);
+			break;
+		}
+
+		g_object_unref (info);
+		g_object_unref (file);
+
+		info = tmp_info;
+		file = tmp_file;
+	}
+
+	uri = g_file_get_uri (file);
 	if (g_file_query_exists (file, NULL)) {
+		const gchar *mime;
+
+		mime = g_file_info_get_content_type (info);
 	  	type = brasero_project_manager_open_by_mime (manager, uri, mime);
         } 
 	else {
-	  	dialog = gtk_message_dialog_new (GTK_WINDOW (window),
-					   	GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-					   	GTK_MESSAGE_ERROR,
-					   	GTK_BUTTONS_CLOSE,
-					   	"Error loading project");
-	  	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-							  _("The project '%s' does not exist."),
-							  uri);
-	  	gtk_dialog_run (GTK_DIALOG (dialog));
-	  	gtk_widget_destroy (dialog);
+		gchar *string;
+
+		string = g_strdup_printf (_("The project '%s' does not exist."), uri);
+		brasero_utils_message_dialog (gtk_widget_get_toplevel (GTK_WIDGET (manager)),
+					      _("Error while loading the project:"),
+					      string,
+					      GTK_MESSAGE_ERROR);
+		g_free (string);
+
+		type = BRASERO_PROJECT_TYPE_INVALID;
 	}
 
 	g_free (uri);
@@ -822,6 +1110,7 @@ brasero_project_manager_open_cb (GtkAction *action, BraseroProjectManager *manag
 	uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (chooser));
 	gtk_widget_destroy (chooser);
 
+	manager->priv->oneshot = FALSE;
 	type = brasero_project_manager_open_uri (manager, uri);
 	g_free (uri);
 }
@@ -831,9 +1120,17 @@ brasero_project_manager_recent_clicked_cb (BraseroProjectTypeChooser *chooser,
 					   const gchar *uri,
 					   BraseroProjectManager *manager)
 {
+	manager->priv->oneshot = FALSE;
 	brasero_project_manager_open_uri (manager, uri);
 }
 
+void
+brasero_project_manager_set_oneshot (BraseroProjectManager *manager,
+				     gboolean oneshot)
+{
+	manager->priv->oneshot = oneshot;
+}
+
 void
 brasero_project_manager_empty (BraseroProjectManager *manager)
 {
@@ -927,6 +1224,11 @@ brasero_project_manager_init (BraseroProjectManager *obj)
 	gtk_widget_show (obj->priv->layout);
 	gtk_notebook_append_page (GTK_NOTEBOOK (obj), obj->priv->layout, NULL);
 
+	g_signal_connect (obj->priv->layout,
+			  "sidepane",
+			  G_CALLBACK (brasero_project_manager_sidepane_changed),
+			  obj);
+
 	/* create the project for audio and data discs */
 	obj->priv->project = brasero_project_new ();
 
@@ -951,7 +1253,7 @@ brasero_project_manager_init (BraseroProjectManager *obj)
 				   _("Browse the file system"),
 				   _("Display File Browser"),
 				   GTK_STOCK_DIRECTORY,
-				   BRASERO_LAYOUT_AUDIO|BRASERO_LAYOUT_DATA);
+				   BRASERO_LAYOUT_AUDIO|BRASERO_LAYOUT_DATA|BRASERO_LAYOUT_VIDEO);
 
 #ifdef BUILD_PREVIEW
 	brasero_preview_add_source (BRASERO_PREVIEW (preview),
@@ -975,7 +1277,7 @@ brasero_project_manager_init (BraseroProjectManager *obj)
 				   _("Search files using keywords"),
 				   _("Display Search"),
 				   GTK_STOCK_FIND,
-				   BRASERO_LAYOUT_AUDIO|BRASERO_LAYOUT_DATA);
+				   BRASERO_LAYOUT_AUDIO|BRASERO_LAYOUT_DATA|BRASERO_LAYOUT_VIDEO);
 
 #ifdef BUILD_PREVIEW
 	brasero_preview_add_source (BRASERO_PREVIEW (preview),
@@ -1029,6 +1331,16 @@ brasero_project_manager_finalize (GObject *object)
 		cobj->priv->io = NULL;
 	}
 
+	if (cobj->priv->preview_id) {
+		g_source_remove (cobj->priv->preview_id);
+		cobj->priv->preview_id = 0;
+	}
+
+	if (cobj->priv->selected) {
+		g_strfreev (cobj->priv->selected);
+		cobj->priv->selected = NULL;
+	}
+
 	g_free (cobj->priv);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
