@@ -6,19 +6,21 @@
 *  <brasero-app@wanadoo.fr>
 ****************************************************************************/
 /*
- *  This program is free software; you can redistribute it and/or modify
+ *  Brasero is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Brasero is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -46,8 +48,7 @@
 G_DEFINE_TYPE (BraseroProjectTypeChooser, brasero_project_type_chooser, GTK_TYPE_EVENT_BOX);
 
 typedef enum {
-	PROJECT_CLICKED_SIGNAL,
-	URI_CLICKED_SIGNAL,
+	RECENT_CLICKED_SIGNAL,
 	CHOSEN_SIGNAL,
 	LAST_SIGNAL
 } BraseroProjectTypeChooserSignalType;
@@ -80,6 +81,11 @@ static ItemDescription items [] = {
 	N_("Create a CD/DVD containing any type of data that can only be read on a computer"),
 	"media-optical-data-new",
 	BRASERO_PROJECT_TYPE_DATA},
+       {N_("<big>_Video project</big>"),
+	N_("Create a video DVD or a SVCD"),
+	N_("Create a video DVD or a SVCD that are readable on TV readers"),
+	"media-optical-video-new",
+	BRASERO_PROJECT_TYPE_VIDEO},
        {N_("<big>Disc _copy</big>"),
 	N_("Create 1:1 copy of a CD/DVD"),
 	N_("Create a 1:1 copy of an audio CD or a data CD/DVD on your hardisk or on another CD/DVD"),
@@ -148,49 +154,40 @@ brasero_project_type_chooser_new_item (BraseroProjectTypeChooser *chooser,
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), 4);
 	gtk_container_add (GTK_CONTAINER (eventbox), vbox);
 
-	hbox = gtk_hbox_new (FALSE, 6);
+	hbox = gtk_hbox_new (FALSE, 4);
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, FALSE, 0);
 
 	image = gtk_image_new_from_icon_name (description->image, GTK_ICON_SIZE_DIALOG);
 	gtk_misc_set_alignment (GTK_MISC (image), 1.0, 0.5);
-	gtk_box_pack_start (GTK_BOX (hbox), image, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
+
+	vbox = gtk_vbox_new (TRUE, 4);
+	gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, TRUE, 0);
 
 	label = gtk_label_new (NULL);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), eventbox);
 	gtk_label_set_markup_with_mnemonic (GTK_LABEL (label), _(description->text));
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-	gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, TRUE, 0);
 	g_object_set_data (G_OBJECT (eventbox), LABEL_KEY, label);
 
 	label = gtk_label_new (NULL);
+	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	gtk_label_set_markup (GTK_LABEL (label), _(description->description));
-	gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, TRUE, 0);
 
 	return eventbox;
 }
 
 static void
-brasero_project_type_chooser_uri_clicked_cb (GtkButton *button,
-					     BraseroProjectTypeChooser *self)
+brasero_project_type_chooser_recent_clicked_cb (GtkButton *button,
+						BraseroProjectTypeChooser *self)
 {
 	const gchar *uri;
 
 	uri = gtk_link_button_get_uri (GTK_LINK_BUTTON (button));
 	g_signal_emit (self,
-		       brasero_project_type_chooser_signals [URI_CLICKED_SIGNAL],
-		       0,
-		       uri);
-}
-
-static void
-brasero_project_type_chooser_project_clicked_cb (GtkButton *button,
-						 BraseroProjectTypeChooser *self)
-{
-	const gchar *uri;
-
-	uri = gtk_link_button_get_uri (GTK_LINK_BUTTON (button));
-	g_signal_emit (self,
-		       brasero_project_type_chooser_signals [PROJECT_CLICKED_SIGNAL],
+		       brasero_project_type_chooser_signals [RECENT_CLICKED_SIGNAL],
 		       0,
 		       uri);
 }
@@ -217,6 +214,7 @@ static void
 brasero_project_type_chooser_build_recent (BraseroProjectTypeChooser *self,
 					   GtkRecentManager *recent)
 {
+	GtkSizeGroup *group;
 	GList *list = NULL;
 	GList *recents;
 	GList *iter;
@@ -236,7 +234,11 @@ brasero_project_type_chooser_build_recent (BraseroProjectTypeChooser *self,
 		&&  strcmp (mime, "application/x-cd-image")
 		&&  strcmp (mime, "application/x-cdrdao-toc")
 		&&  strcmp (mime, "application/x-toc")
-		&&  strcmp (mime, "application/x-cue"))
+		&&  strcmp (mime, "application/x-cue")
+		&&  strcmp (mime, "audio/x-scpls")
+		&&  strcmp (mime, "audio/x-ms-asx")
+		&&  strcmp (mime, "audio/x-mp3-playlist")
+		&&  strcmp (mime, "audio/x-mpegurl"))
 			continue;
 
 		/* sort */
@@ -244,9 +246,10 @@ brasero_project_type_chooser_build_recent (BraseroProjectTypeChooser *self,
 					     info,
 					     brasero_project_type_chooser_sort_recent);
 		if (g_list_length (list) > 5)
-			list = g_list_remove_link (list, g_list_last (list));
+			list = g_list_delete_link (list, g_list_last (list));
 	}
 
+	group = gtk_size_group_new (GTK_SIZE_GROUP_BOTH);
 	for (iter = list; iter; iter = iter->next) {
 		GtkRecentInfo *info;
 		const gchar *name;
@@ -264,7 +267,7 @@ brasero_project_type_chooser_build_recent (BraseroProjectTypeChooser *self,
 		gtk_box_pack_start (GTK_BOX (self->priv->recent_box),
 				    hbox,
 				    FALSE,
-				    FALSE,
+				    TRUE,
 				    0);
 
 		tooltip = gtk_recent_info_get_uri_display (info);
@@ -280,30 +283,33 @@ brasero_project_type_chooser_build_recent (BraseroProjectTypeChooser *self,
 		uri = gtk_recent_info_get_uri (info);
 
 		link = gtk_link_button_new_with_label (uri, name);
+		gtk_button_set_alignment (GTK_BUTTON (link), 0.0, 0.5);
 		gtk_button_set_focus_on_click (GTK_BUTTON (link), FALSE);
 		gtk_button_set_image (GTK_BUTTON (link), image);
-		if (strcmp (gtk_recent_info_get_mime_type (info), "application/x-brasero"))
-			g_signal_connect (link,
-					  "clicked",
-					  G_CALLBACK (brasero_project_type_chooser_uri_clicked_cb),
-					  self);
-		else
-			g_signal_connect (link,
-					  "clicked",
-					  G_CALLBACK (brasero_project_type_chooser_project_clicked_cb),
-					  self);
+		g_signal_connect (link,
+				  "clicked",
+				  G_CALLBACK (brasero_project_type_chooser_recent_clicked_cb),
+				  self);
 
 		gtk_widget_show (link);
 		gtk_widget_set_tooltip_text (link, tooltip);
-		gtk_box_pack_start (GTK_BOX (hbox), link, FALSE, FALSE, 0);
+		gtk_box_pack_start (GTK_BOX (hbox), link, FALSE, TRUE, 0);
 
 		g_free (tooltip);
+
+		gtk_size_group_add_widget (group, link);
 	}
+	g_object_unref (group);
 
 	if (!g_list_length (list)) {
 		GtkWidget *label;
+		gchar *string;
+
+		string = g_strdup_printf ("<i>%s</i>", _("No recently used project"));
+		label = gtk_label_new (string);
+		gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+		g_free (string);
 
-		label = gtk_label_new (_("No recently used project"));
 		gtk_widget_show (label);
 		gtk_box_pack_start (GTK_BOX (self->priv->recent_box), label, FALSE, FALSE, 0);
 	}
@@ -331,11 +337,13 @@ brasero_project_type_chooser_init (BraseroProjectTypeChooser *obj)
 	GtkWidget *project_box;
 	GtkWidget *recent_box;
 	GError *error = NULL;
+	GtkWidget *separator;
 	GtkWidget *widget;
 	GtkWidget *table;
 	GtkWidget *label;
 	GtkWidget *vbox;
-	int nb_rows = 2;
+	int nb_rows = 1;
+	gchar *string;
 	int nb_items;
 	int rows;
 	int i;
@@ -349,25 +357,22 @@ brasero_project_type_chooser_init (BraseroProjectTypeChooser *obj)
 		error = NULL;
 	}
 
-	vbox = gtk_vbox_new (FALSE, 0);
+	vbox = gtk_hbox_new (FALSE, 0);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
 	gtk_widget_show (vbox);
 	gtk_container_add (GTK_CONTAINER (obj), vbox);
 
 	project_box = gtk_vbox_new (FALSE, 6);
-	gtk_container_set_border_width (GTK_CONTAINER (project_box), 12);
 	gtk_widget_show (project_box);
-	gtk_box_pack_start (GTK_BOX (vbox), project_box, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), project_box, FALSE, TRUE, 0);
+
+	string = g_strdup_printf ("<span size='x-large'><b>%s</b></span>", _("Create a new project:"));
+	label = gtk_label_new (string);
+	g_free (string);
 
-	label = gtk_label_new (_("<span size='x-large'><b>Create a new project:</b></span>"));
 	gtk_widget_show (label);
 	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
-	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-	gtk_box_pack_start (GTK_BOX (project_box), label, FALSE, FALSE, 0);
-
-	label = gtk_label_new (_("<span foreground='grey50'><b><i>Choose from the following options</i></b></span>"));
-	gtk_widget_show (label);
-	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
-	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.0);
 	gtk_box_pack_start (GTK_BOX (project_box), label, FALSE, FALSE, 0);
 
 	/* get the number of rows */
@@ -377,11 +382,11 @@ brasero_project_type_chooser_init (BraseroProjectTypeChooser *obj)
 		rows ++;
 
 	table = gtk_table_new (rows, nb_rows, TRUE);
-	gtk_container_set_border_width (GTK_CONTAINER (table), 12);
-	gtk_box_pack_start (GTK_BOX (project_box), table, FALSE, FALSE, 6);
+	gtk_container_set_border_width (GTK_CONTAINER (table), 6);
+	gtk_box_pack_start (GTK_BOX (project_box), table, FALSE, TRUE, 0);
 
-	gtk_table_set_col_spacings (GTK_TABLE (table), 12);
-	gtk_table_set_row_spacings (GTK_TABLE (table), 12);
+	gtk_table_set_col_spacings (GTK_TABLE (table), 4);
+	gtk_table_set_row_spacings (GTK_TABLE (table), 4);
 
 	for (i = 0; i < nb_items; i ++) {
 		widget = brasero_project_type_chooser_new_item (obj, items + i);
@@ -398,22 +403,28 @@ brasero_project_type_chooser_init (BraseroProjectTypeChooser *obj)
 	}
 	gtk_widget_show_all (table);
 
+	separator = gtk_vseparator_new ();
+	gtk_widget_show (separator);
+	gtk_box_pack_start (GTK_BOX (vbox), separator, FALSE, TRUE, 8);
+
 	/* The recent files part */
 	recent_box = gtk_vbox_new (FALSE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (recent_box), 12);
 	gtk_widget_show (recent_box);
-	gtk_box_pack_start (GTK_BOX (vbox), recent_box, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), recent_box, FALSE, TRUE, 0);
+
+	string = g_strdup_printf ("<span size='x-large'><b>%s</b></span>", _("Recent projects:"));
+	label = gtk_label_new (string);
+	g_free (string);
 
-	label = gtk_label_new (_("<span size='x-large'><b>Choose a recently opened project:</b></span>"));
 	gtk_widget_show (label);
 	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
-	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-	gtk_box_pack_start (GTK_BOX (recent_box), label, FALSE, FALSE, 12);
+	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.0);
+	gtk_box_pack_start (GTK_BOX (recent_box), label, FALSE, FALSE, 0);
 
-	vbox = gtk_vbox_new (FALSE, 6);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
+	vbox = gtk_vbox_new (TRUE, 0);
 	gtk_widget_show (vbox);
-	gtk_box_pack_start (GTK_BOX (recent_box), vbox, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (recent_box), vbox, FALSE, TRUE, 0);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
 	obj->priv->recent_box = vbox;
 
 	recent = gtk_recent_manager_get_default ();
@@ -496,19 +507,10 @@ brasero_project_type_chooser_class_init (BraseroProjectTypeChooserClass *klass)
 			  G_TYPE_NONE,
 			  1,
 			  G_TYPE_UINT);
-	brasero_project_type_chooser_signals[URI_CLICKED_SIGNAL] =
-	    g_signal_new ("uri_clicked", G_OBJECT_CLASS_TYPE (object_class),
+	brasero_project_type_chooser_signals[RECENT_CLICKED_SIGNAL] =
+	    g_signal_new ("recent_clicked", G_OBJECT_CLASS_TYPE (object_class),
 			  G_SIGNAL_ACTION | G_SIGNAL_RUN_FIRST,
-			  G_STRUCT_OFFSET (BraseroProjectTypeChooserClass, uri_clicked),
-			  NULL, NULL,
-			  g_cclosure_marshal_VOID__STRING,
-			  G_TYPE_NONE,
-			  1,
-			  G_TYPE_STRING);
-	brasero_project_type_chooser_signals[PROJECT_CLICKED_SIGNAL] =
-	    g_signal_new ("project_clicked", G_OBJECT_CLASS_TYPE (object_class),
-			  G_SIGNAL_ACTION | G_SIGNAL_RUN_FIRST,
-			  G_STRUCT_OFFSET (BraseroProjectTypeChooserClass, project_clicked),
+			  G_STRUCT_OFFSET (BraseroProjectTypeChooserClass, recent_clicked),
 			  NULL, NULL,
 			  g_cclosure_marshal_VOID__STRING,
 			  G_TYPE_NONE,
