@@ -7,19 +7,21 @@
  ***************************************************************************/
 
 /*
- *  This program is free software; you can redistribute it and/or modify
+ *  Brasero is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Brasero is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -111,12 +113,12 @@ brasero_sum_dialog_message (BraseroSumDialog *self,
 					  GTK_DIALOG_DESTROY_WITH_PARENT,
 					  type,
 					  GTK_BUTTONS_NONE,
-					  primary_message);
+					  "%s", primary_message);
 
 	gtk_window_set_title (GTK_WINDOW (message), title);
 
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message),
-						  secondary_message);
+						  "%s", secondary_message);
 
 	button = brasero_utils_make_button (_("Check _Again"),
 					    GTK_STOCK_FIND,
@@ -149,7 +151,7 @@ brasero_sum_dialog_message_error (BraseroSumDialog *self,
 					NULL);
 
 	return brasero_sum_dialog_message (self,
-					   _("File integrity check error"),
+					   _("File Integrity Check Error"),
 					   _("The file integrity check cannot be performed:"),
 					   error ? error->message:_("unknown error"),
 					   GTK_MESSAGE_ERROR);
@@ -163,7 +165,7 @@ brasero_sum_dialog_success (BraseroSumDialog *self)
 					NULL);
 
 	return brasero_sum_dialog_message (self,
-					   _("File integrity check success"),
+					   _("File Integrity Check Success"),
 					   _("The file integrity was performed successfully:"),
 					   _("there seems to be no corrupted file on the disc."),
 					   GTK_MESSAGE_INFO);
@@ -176,9 +178,9 @@ enum {
 
 static gboolean
 brasero_sum_dialog_corruption_warning (BraseroSumDialog *self,
-				       GSList *wrong_sums)
+				       const gchar **wrong_sums)
 {
-	GSList *iter;
+	gchar *string;
 	GtkWidget *tree;
 	GtkWidget *scroll;
 	GtkWidget *button;
@@ -188,14 +190,16 @@ brasero_sum_dialog_corruption_warning (BraseroSumDialog *self,
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *column;
 
+	string = g_strdup_printf ("<b><big>%s</big></b>", _("The following files appear to be corrupted:"));
 	message = gtk_message_dialog_new_with_markup (GTK_WINDOW (self),
 						      GTK_DIALOG_MODAL |
 						      GTK_DIALOG_DESTROY_WITH_PARENT,
 						      GTK_MESSAGE_ERROR,
 						      GTK_BUTTONS_NONE,
-						      _("<b><big>The following files appear to be corrupted:</big></b>"));
+						      "%s", string);
+	g_free (string);
 
-	gtk_window_set_title (GTK_WINDOW (message),  _("File integrity check error"));
+	gtk_window_set_title (GTK_WINDOW (message),  _("File Integrity Check Error"));
 	gtk_window_set_resizable (GTK_WINDOW (message), TRUE);
 	gtk_widget_set_size_request (GTK_WIDGET (message), 440, 300);
 
@@ -214,11 +218,11 @@ brasero_sum_dialog_corruption_warning (BraseroSumDialog *self,
 
 	/* build a list */
 	model = GTK_TREE_MODEL (gtk_list_store_new (BRASERO_SUM_DIALOG_NB_COL, G_TYPE_STRING));
-	for (iter = wrong_sums; iter; iter = iter->next) {
-		gchar *path;
+	for (; wrong_sums && (*wrong_sums); wrong_sums ++) {
+		const gchar *path;
 		GtkTreeIter tree_iter;
 
-		path = iter->data;
+		path = (*wrong_sums);
 		gtk_list_store_append (GTK_LIST_STORE (model), &tree_iter);
 		gtk_list_store_set (GTK_LIST_STORE (model), &tree_iter,
 				    BRASERO_SUM_DIALOG_PATH, path,
@@ -394,6 +398,8 @@ brasero_sum_dialog_get_file_checksum (BraseroSumDialog *self,
 	/* now get the md5 sum from the file */
 	file = fopen (src, "r");
 	if (!file) {
+                int errsv = errno;
+
 		if (tmppath)
 			g_remove (tmppath);
 
@@ -402,7 +408,8 @@ brasero_sum_dialog_get_file_checksum (BraseroSumDialog *self,
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
-			     strerror (errno));
+			     "%s",
+			     g_strerror (errsv));
 		return BRASERO_BURN_ERR;
 	}
 
@@ -419,7 +426,8 @@ brasero_sum_dialog_get_file_checksum (BraseroSumDialog *self,
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
-			     strerror (errno));
+			     "%s",
+			     g_strerror (errno));
 
 		fclose (file);
 		return BRASERO_BURN_ERR;
@@ -470,7 +478,7 @@ brasero_sum_dialog_check_md5_file (BraseroSumDialog *self,
     	uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (self->priv->md5_chooser));
 	if (!uri) {
 		retval = brasero_sum_dialog_message (self,
-						     _("File integrity check error"),
+						     _("File Integrity Check Error"),
 						     _("The file integrity check cannot be performed:"),
 						     error ? error->message:_("no md5 file was given."),
 						     GTK_MESSAGE_ERROR);
@@ -537,8 +545,10 @@ brasero_sum_dialog_set_track_checksum_type (BraseroSumDialog *self,
 		return BRASERO_CHECKSUM_NONE;
 
 	filename = g_build_path (G_DIR_SEPARATOR_S, root, BRASERO_MD5_FILE, NULL);
+
 	if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
 		g_free (filename);
+		g_free (root);
 
 		brasero_track_set_checksum (track,
 					    BRASERO_CHECKSUM_MD5_FILE,
@@ -551,6 +561,7 @@ brasero_sum_dialog_set_track_checksum_type (BraseroSumDialog *self,
 	filename = g_build_path (G_DIR_SEPARATOR_S, root, BRASERO_SHA1_FILE, NULL);
 	if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
 		g_free (filename);
+		g_free (root);
 
 		brasero_track_set_checksum (track,
 					    BRASERO_CHECKSUM_SHA1_FILE,
@@ -563,6 +574,7 @@ brasero_sum_dialog_set_track_checksum_type (BraseroSumDialog *self,
 	filename = g_build_path (G_DIR_SEPARATOR_S, root, BRASERO_SHA256_FILE, NULL);
 	if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
 		g_free (filename);
+		g_free (root);
 
 		brasero_track_set_checksum (track,
 					    BRASERO_CHECKSUM_SHA256_FILE,
@@ -571,6 +583,7 @@ brasero_sum_dialog_set_track_checksum_type (BraseroSumDialog *self,
 		return BRASERO_CHECKSUM_SHA256_FILE;
 	}
 	g_free (filename);
+	g_free (root);
 
 	g_set_error (error,
 		     BRASERO_ERROR,
@@ -585,9 +598,9 @@ brasero_sum_dialog_check_disc_sum (BraseroSumDialog *self,
 				   BraseroDrive *drive)
 {
 	BraseroChecksumType checksum_type;
-	GSList *wrong_sums = NULL;
 	BraseroBurnResult result;
 	GError *error = NULL;
+	GValue *value = NULL;
 	BraseroTrack *track;
 	BraseroBurn *burn;
 	gboolean retval;
@@ -608,6 +621,9 @@ brasero_sum_dialog_check_disc_sum (BraseroSumDialog *self,
 		return FALSE;
 	}
 
+	/* no eject at the end (it's default) */
+	brasero_burn_session_remove_flag (self->priv->session, BRASERO_BURN_FLAG_EJECT);
+
 	brasero_burn_session_add_track (self->priv->session, track);
 
 	/* It's good practice to unref the track afterwards as we don't need it
@@ -638,12 +654,11 @@ brasero_sum_dialog_check_disc_sum (BraseroSumDialog *self,
 
 	g_error_free (error);
 
-	wrong_sums = brasero_burn_session_get_wrong_checksums (self->priv->session);
-	retval = brasero_sum_dialog_corruption_warning (self, wrong_sums);
-	g_slist_foreach (wrong_sums, (GFunc) g_free, NULL);
-	g_slist_free (wrong_sums);
+	brasero_track_tag_lookup (track,
+				  BRASERO_TRACK_MEDIUM_WRONG_CHECKSUM_TAG,
+				  &value);
 
-	return retval;
+	return brasero_sum_dialog_corruption_warning (self, g_value_get_boxed (value));
 }
 
 static gboolean
@@ -750,6 +765,10 @@ brasero_sum_dialog_init (BraseroSumDialog *obj)
 					GTK_STOCK_FIND,
 					NULL);
 
+	/* only media with data, no blank medium */
+	brasero_tool_dialog_set_medium_type_shown (BRASERO_TOOL_DIALOG (obj),
+						   BRASERO_MEDIA_TYPE_READABLE);
+
 	medium = brasero_tool_dialog_get_medium (BRASERO_TOOL_DIALOG (obj));
 	if (medium) {
 		brasero_tool_dialog_set_valid (BRASERO_TOOL_DIALOG (obj), BRASERO_MEDIUM_VALID (brasero_medium_get_status (medium)));
@@ -765,7 +784,7 @@ brasero_sum_dialog_new ()
 	BraseroSumDialog *obj;
 	
 	obj = BRASERO_SUM_DIALOG (g_object_new (BRASERO_TYPE_SUM_DIALOG, NULL));
-	gtk_window_set_title (GTK_WINDOW (obj), "Disc checking");
+	gtk_window_set_title (GTK_WINDOW (obj), "Disc Checking");
 	
 	return GTK_WIDGET (obj);
 }
