@@ -7,19 +7,21 @@
  ***************************************************************************/
 
 /*
- *  This program is free software; you can redistribute it and/or modify
+ *  Brasero is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Brasero is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.
  */
 
 #ifndef BRASERO_TOOL_DIALOG_H
@@ -35,6 +37,7 @@
 #include "burn.h"
 #include "burn-medium.h"
 #include "burn-session.h"
+#include "burn-medium-monitor.h"
 
 G_BEGIN_DECLS
 
@@ -58,9 +61,10 @@ struct _BraseroToolDialogClass {
 	GtkDialogClass parent_class;
 
 	gboolean	(*activate)		(BraseroToolDialog *dialog,
-						 NautilusBurnDrive *drive);
+						 BraseroMedium *medium);
+	void		(*cancel)		(BraseroToolDialog *dialog);
 	void		(*drive_changed)	(BraseroToolDialog *dialog,
-						 NautilusBurnDrive *drive);
+						 BraseroMedium *medium);
 };
 
 GType brasero_tool_dialog_get_type ();
@@ -76,24 +80,30 @@ brasero_tool_dialog_set_button (BraseroToolDialog *dialog,
 void
 brasero_tool_dialog_set_valid (BraseroToolDialog *dialog,
 			       gboolean valid);
-
 void
-brasero_tool_dialog_set_progress (BraseroToolDialog *self,
+brasero_tool_dialog_set_medium_type_shown (BraseroToolDialog *dialog,
+					   BraseroMediaType media_type);
+void
+brasero_tool_dialog_set_progress (BraseroToolDialog *dialog,
 				  gdouble overall_progress,
 				  gdouble task_progress,
 				  glong remaining,
 				  gint size_mb,
 				  gint written_mb);
 void
-brasero_tool_dialog_set_action (BraseroToolDialog *self,
+brasero_tool_dialog_set_action (BraseroToolDialog *dialog,
 				BraseroBurnAction action,
 				const gchar *string);
 
 BraseroBurn *
 brasero_tool_dialog_get_burn (BraseroToolDialog *dialog);
 
-NautilusBurnDrive *
-brasero_tool_dialog_get_drive (BraseroToolDialog *dialog);
+BraseroMedium *
+brasero_tool_dialog_get_medium (BraseroToolDialog *dialog);
+
+void
+brasero_tool_dialog_set_medium (BraseroToolDialog *dialog,
+				BraseroMedium *medium);
 
 G_END_DECLS
 
