@@ -106,14 +106,14 @@ brasero_debug_burn_flags_to_string (gchar *buffer,
 		strcat (buffer, "no grace, ");
 	if (flags & BRASERO_BURN_FLAG_DAO)
 		strcat (buffer, "dao, ");
+	if (flags & BRASERO_BURN_FLAG_RAW)
+		strcat (buffer, "raw, ");
 	if (flags & BRASERO_BURN_FLAG_OVERBURN)
 		strcat (buffer, "overburn, ");
 	if (flags & BRASERO_BURN_FLAG_BURNPROOF)
 		strcat (buffer, "burnproof, ");
 	if (flags & BRASERO_BURN_FLAG_NO_TMP_FILES)
 		strcat (buffer, "no tmp file, ");
-	if (flags & BRASERO_BURN_FLAG_DONT_CLEAN_OUTPUT)
-		strcat (buffer, "clean output, ");
 	if (flags & BRASERO_BURN_FLAG_DONT_OVERWRITE)
 		strcat (buffer, "no overwrite, ");
 	if (flags & BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE)
@@ -176,12 +176,12 @@ brasero_debug_medium_info_to_string (gchar *buffer,
 		strcat (buffer, "DVD ");
 
 	if (media & BRASERO_MEDIUM_RAM)
-		strcat (buffer, "RAM");
+		strcat (buffer, "RAM ");
 
 	if (media & BRASERO_MEDIUM_BD)
 		strcat (buffer, "BD ");
 
-	if (media & BRASERO_MEDIUM_DVD_DL)
+	if (media & BRASERO_MEDIUM_DUAL_L)
 		strcat (buffer, "DL ");
 
 	/* DVD subtypes */
@@ -197,6 +197,16 @@ brasero_debug_medium_info_to_string (gchar *buffer,
 	if (media & BRASERO_MEDIUM_JUMP)
 		strcat (buffer, "- (jump) ");
 
+	/* BD subtypes */
+	if (media & BRASERO_MEDIUM_SRM)
+		strcat (buffer, "SRM ");
+
+	if (media & BRASERO_MEDIUM_POW)
+		strcat (buffer, "POW ");
+
+	if (media & BRASERO_MEDIUM_RANDOM)
+		strcat (buffer, "RANDOM ");
+
 	/* discs attributes */
 	if (media & BRASERO_MEDIUM_REWRITABLE)
 		strcat (buffer, "RW ");
@@ -258,6 +268,8 @@ brasero_debug_data_fs_to_string (gchar *buffer,
 		strcat (buffer, "JOLIET ");
 	if (fs_type & BRASERO_IMAGE_FS_VIDEO)
 		strcat (buffer, "VIDEO ");
+	if (fs_type & BRASERO_IMAGE_ISO_FS_DEEP_DIRECTORY)
+		strcat (buffer, "DEEP ");
 }
 
 static void
