@@ -2,17 +2,17 @@
  *            burn-track.h
  *
  *  Thu Dec  7 09:51:03 2006
- *  Copyright  2006  algernon
- *  <algernon@localhost.localdomain>
+ *  Copyright  2006  Rouquier Philippe
+ *  <bonfire-app@wanadoo.fr>
  ****************************************************************************/
 
 /*
- * This program is free software; you can redistribute it and/or modify
+ * Brasero is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  * 
- * This program is distributed in the hope that it will be useful,
+ * Brasero is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU Library General Public License for more details.
@@ -63,9 +63,19 @@ typedef enum {
 	BRASERO_AUDIO_FORMAT_NONE		= 0,
 	BRASERO_AUDIO_FORMAT_UNDEFINED		= 1,
 	BRASERO_AUDIO_FORMAT_4_CHANNEL		= 1 << 1,
-	BRASERO_AUDIO_FORMAT_RAW		= 1 << 2
+	BRASERO_AUDIO_FORMAT_RAW		= 1 << 2,
+	BRASERO_AUDIO_FORMAT_AC3		= 1 << 3,
+	BRASERO_AUDIO_FORMAT_MP2		= 1 << 4,
+	BRASERO_AUDIO_FORMAT_44100		= 1 << 5,
+	BRASERO_AUDIO_FORMAT_48000		= 1 << 6,
+	BRASERO_VIDEO_FORMAT_UNDEFINED		= 1 << 7,
+	BRASERO_VIDEO_FORMAT_VCD		= 1 << 8,
+	BRASERO_VIDEO_FORMAT_VIDEO_DVD		= 1 << 9
 } BraseroAudioFormat;
 
+#define BRASERO_AUDIO_CAPS_AUDIO(caps_FORMAT)	((caps_FORMAT) & 0x007F)
+#define BRASERO_AUDIO_CAPS_VIDEO(caps_FORMAT)	((caps_FORMAT) & 0x0180)
+
 typedef enum {
 	BRASERO_CHECKSUM_NONE			= 0,
 	BRASERO_CHECKSUM_MD5			= 1,
@@ -135,6 +145,16 @@ struct _BraseroTrackType {
 };
 typedef struct _BraseroTrackType BraseroTrackType;
 
+BraseroBurnResult
+brasero_track_type_is_supported (BraseroTrackType *type);
+
+
+#define BRASERO_TRACK_TYPE_HAS_VIDEO(type_MACRO)				\
+	(((type_MACRO)->type == BRASERO_TRACK_TYPE_AUDIO) &&			\
+	 ((type_MACRO)->subtype.audio_format & (BRASERO_VIDEO_FORMAT_UNDEFINED|	\
+						BRASERO_VIDEO_FORMAT_VCD|	\
+						BRASERO_VIDEO_FORMAT_VIDEO_DVD)))
+
 gboolean
 brasero_track_type_equal (const BraseroTrackType *type_A,
 			  const BraseroTrackType *type_B);
@@ -239,6 +259,7 @@ brasero_track_get_data_paths (BraseroTrack *track,
 			      const gchar *grafts_path,
 			      const gchar *excluded_path,
 			      const gchar *emptydir,
+			      const gchar *videodir,
 			      GError **error);
 
 gchar *
@@ -306,6 +327,19 @@ brasero_track_tag_lookup (BraseroTrack *track,
 			  const gchar *tag,
 			  GValue **value);
 
+/*
+ * Commonly used Tags
+ */
+
+#define BRASERO_TRACK_MEDIUM_ADDRESS_START_TAG		"track::medium::address::start"
+#define BRASERO_TRACK_MEDIUM_ADDRESS_END_TAG		"track::medium::address::end"
+
+/**
+ * Array of filenames (on medium) which have a wrong checksum value (G_TYPE_STRV)
+ */
+
+#define BRASERO_TRACK_MEDIUM_WRONG_CHECKSUM_TAG		"track::medium::error::checksum::list"
+
 G_END_DECLS
 
 #endif /* _BURN_TRACK_H */
