@@ -1,26 +1,23 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /*
  * brasero
- * Copyright (C) Philippe Rouquier 2008 <bonfire-app@wanadoo.fr>
+ * Copyright (C) Philippe Rouquier 2005-2008 <bonfire-app@wanadoo.fr>
+ *
+ * Brasero is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  * 
- * brasero is free software.
- * 
- * You may redistribute it and/or modify it under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option)
- * any later version.
- * 
- * brasero is distributed in the hope that it will be useful,
+ * Brasero is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Library General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
- * along with brasero.  If not, write to:
- * 	The Free Software Foundation, Inc.,
- * 	51 Franklin Street, Fifth Floor
- * 	Boston, MA  02110-1301, USA.
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
  */
+
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
@@ -33,6 +30,7 @@
 #include <gio/gio.h>
 
 #include "burn-basics.h"
+#include "burn-debug.h"
 #include "burn-volume-obj.h"
 
 typedef struct _BraseroVolumePrivate BraseroVolumePrivate;
@@ -74,7 +72,12 @@ brasero_volume_get_gvolume (BraseroVolume *self)
 	priv = BRASERO_VOLUME_PRIVATE (self);
 
 	drive = brasero_medium_get_drive (BRASERO_MEDIUM (self));
+
+#if defined(HAVE_STRUCT_USCSI_CMD)
+	volume_path = brasero_drive_get_block_device (drive);
+#else
 	volume_path = brasero_drive_get_device (drive);
+#endif
 
 	/* NOTE: medium-monitor already holds a reference for GVolumeMonitor */
 	monitor = g_volume_monitor_get ();
@@ -87,7 +90,10 @@ brasero_volume_get_gvolume (BraseroVolume *self)
 
 		tmp = iter->data;
 		device_path = g_volume_get_identifier (tmp, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
+		if (!device_path)
+			continue;
 
+		BRASERO_BURN_LOG ("Found volume %s", device_path);
 		if (!strcmp (device_path, volume_path)) {
 			volume = tmp;
 			g_free (device_path);
@@ -101,11 +107,65 @@ brasero_volume_get_gvolume (BraseroVolume *self)
 	g_list_free (volumes);
 
 	if (!volume)
-		g_warning ("No volume found for medium");
+		BRASERO_BURN_LOG ("No volume found for medium");
 
 	return volume;
 }
 
+static GDrive *
+brasero_volume_get_gdrive (BraseroVolume *self)
+{
+	const gchar *volume_path = NULL;
+	BraseroVolumePrivate *priv;
+	GVolumeMonitor *monitor;
+	GDrive *gdrive = NULL;
+	BraseroDrive *drive;
+	GList *drives;
+	GList *iter;
+
+	priv = BRASERO_VOLUME_PRIVATE (self);
+
+	drive = brasero_medium_get_drive (BRASERO_MEDIUM (self));
+
+#if defined(HAVE_STRUCT_USCSI_CMD)
+	volume_path = brasero_drive_get_block_device (drive);
+#else
+	volume_path = brasero_drive_get_device (drive);
+#endif
+
+	/* NOTE: medium-monitor already holds a reference for GVolumeMonitor */
+	monitor = g_volume_monitor_get ();
+	drives = g_volume_monitor_get_connected_drives (monitor);
+	g_object_unref (monitor);
+
+	for (iter = drives; iter; iter = iter->next) {
+		gchar *device_path;
+		GDrive *tmp;
+
+		tmp = iter->data;
+		device_path = g_drive_get_identifier (tmp, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
+		if (!device_path)
+			continue;
+
+		BRASERO_BURN_LOG ("Found drive %s", device_path);
+		if (!strcmp (device_path, volume_path)) {
+			gdrive = tmp;
+			g_free (device_path);
+			g_object_ref (gdrive);
+			break;
+		}
+
+		g_free (device_path);
+	}
+	g_list_foreach (drives, (GFunc) g_object_unref, NULL);
+	g_list_free (drives);
+
+	if (!drive)
+		BRASERO_BURN_LOG ("No drive found for medium");
+
+	return gdrive;
+}
+
 gboolean
 brasero_volume_is_mounted (BraseroVolume *self)
 {
@@ -119,6 +179,9 @@ brasero_volume_is_mounted (BraseroVolume *self)
 	priv = BRASERO_VOLUME_PRIVATE (self);
 
 	volume = brasero_volume_get_gvolume (self);
+	if (!volume)
+		return FALSE;
+
 	if (!g_volume_can_mount (volume)) {
 		/* if it can't be mounted then it's unmounted ... */
 		g_object_unref (volume);
@@ -147,6 +210,8 @@ brasero_volume_get_mount_point (BraseroVolume *self,
 	priv = BRASERO_VOLUME_PRIVATE (self);
 
 	volume = brasero_volume_get_gvolume (self);
+	if (!volume)
+		return NULL;
 
 	/* get the uri for the mount point */
 	mount = g_volume_get_mount (volume);
@@ -296,6 +361,9 @@ brasero_volume_umount (BraseroVolume *self,
 	priv = BRASERO_VOLUME_PRIVATE (self);
 
 	volume = brasero_volume_get_gvolume (self);
+	if (!volume)
+		return TRUE;
+
 	if (!g_volume_can_mount (volume)) {
 		/* if it can't be mounted then it's unmounted ... */
 		g_object_unref (volume);
@@ -395,6 +463,9 @@ brasero_volume_mount (BraseroVolume *self,
 	priv = BRASERO_VOLUME_PRIVATE (self);
 
 	volume = brasero_volume_get_gvolume (self);
+	if (!volume)
+		return FALSE;
+
 	if (!g_volume_can_mount (volume)) {
 		g_object_unref (volume);
 		return FALSE;
@@ -447,9 +518,14 @@ brasero_volume_eject_finish (GObject *source,
 	BraseroVolumePrivate *priv;
 
 	priv = BRASERO_VOLUME_PRIVATE (self);
-	priv->result = g_drive_eject_finish (G_DRIVE (source),
-					     result,
-					     &priv->error);
+	if (G_IS_DRIVE (source))
+		priv->result = g_drive_eject_finish (G_DRIVE (source),
+						     result,
+						     &priv->error);
+	else
+		priv->result = g_volume_eject_finish (G_VOLUME (source),
+						      result,
+						      &priv->error);
 
 	if (priv->error) {
 		if (priv->error->code == G_IO_ERROR_FAILED_HANDLED) {
@@ -465,28 +541,82 @@ brasero_volume_eject_finish (GObject *source,
 		brasero_volume_operation_end (self);
 }
 
+gboolean
+brasero_volume_eject_gvolume (BraseroVolume *self,
+			      gboolean wait,
+			      GVolume *volume,
+			      GError **error)
+{
+	BraseroVolumePrivate *priv;
+	gboolean result;
+
+	priv = BRASERO_VOLUME_PRIVATE (self);
+
+	if (!g_volume_can_eject (volume))
+		return FALSE;
+
+	if (wait) {
+		gulong eject_sig;
+		BraseroDrive *drive;
+
+		drive = brasero_medium_get_drive (BRASERO_MEDIUM (self));
+		eject_sig = g_signal_connect (drive,
+					      "medium-removed",
+					      G_CALLBACK (brasero_volume_ejected_cb),
+					      self);
+
+		g_volume_eject (volume,
+				G_MOUNT_UNMOUNT_NONE,
+				priv->cancel,
+				brasero_volume_eject_finish,
+				self);
+
+		g_object_ref (self);
+		result = brasero_volume_wait_for_operation_end (self, error);
+		g_object_unref (self);
+
+		/* NOTE: from this point on self is no longer valid */
+
+		g_signal_handler_disconnect (drive, eject_sig);
+	}
+	else {
+		g_volume_eject (volume,
+				G_MOUNT_UNMOUNT_NONE,
+				priv->cancel,
+				NULL,
+				self);
+		result = TRUE;
+	}
+
+	return result;
+}
+
 gboolean
 brasero_volume_eject (BraseroVolume *self,
 		      gboolean wait,
 		      GError **error)
 {
 	GDrive *gdrive;
-	gboolean result;
 	GVolume *volume;
+	gboolean result;
 	BraseroVolumePrivate *priv;
 
+	BRASERO_BURN_LOG ("Ejecting");
+
 	if (!self)
 		return TRUE;
 
 	priv = BRASERO_VOLUME_PRIVATE (self);
 
-	volume = brasero_volume_get_gvolume (self);
-	gdrive = g_volume_get_drive (volume);
-	g_object_unref (volume);
+	gdrive = brasero_volume_get_gdrive (self);
+	if (!gdrive) {
+		BRASERO_BURN_LOG ("No GDrive");
+		goto last_resort;
+	}
 
 	if (!g_drive_can_eject (gdrive)) {
-		g_object_unref (gdrive);
-		return FALSE;
+		BRASERO_BURN_LOG ("GDrive can't eject");
+		goto last_resort;
 	}
 
 	if (wait) {
@@ -524,6 +654,18 @@ brasero_volume_eject (BraseroVolume *self,
 
 	g_object_unref (gdrive);
 	return result;
+
+last_resort:
+
+	/* last resort */
+	volume = brasero_volume_get_gvolume (self);
+	result = brasero_volume_eject_gvolume (self, wait, volume, error);
+	g_object_unref (volume);
+
+	if (gdrive)
+		g_object_unref (gdrive);
+
+	return result;
 }
 
 void
@@ -545,34 +687,8 @@ brasero_volume_get_name (BraseroVolume *self)
 {
 	BraseroVolumePrivate *priv;
 	BraseroMedia media;
-	GVolume *volume;
-	gchar *name;
-
-	priv = BRASERO_VOLUME_PRIVATE (self);
-
-	media = brasero_medium_get_status (BRASERO_MEDIUM (self));
-	if (media & BRASERO_MEDIUM_FILE) {
-		/* FIXME: here let's read the image label ?*/
-		return NULL;
-	}
-
-	volume = brasero_volume_get_gvolume (self);
-	name = g_volume_get_name (volume);
-	g_object_unref (volume);
-
-	return name;
-}
-
-gchar *
-brasero_volume_get_display_label (BraseroVolume *self,
-				  gboolean with_markup)
-{
-	BraseroVolumePrivate *priv;
-	BraseroDrive *drive;
-	BraseroMedia media;
 	const gchar *type;
 	GVolume *volume;
-	gchar *label;
 	gchar *name;
 
 	priv = BRASERO_VOLUME_PRIVATE (self);
@@ -582,106 +698,53 @@ brasero_volume_get_display_label (BraseroVolume *self,
 		/* Translators: This is a fake drive, a file, and means that
 		 * when we're writing, we're writing to a file and create an
 		 * image on the hard drive. */
-		label = g_strdup (_("Image File"));
-		if (!with_markup)
-			return label;
-
-		name = label;
-		label = g_strdup_printf ("<b>%s</b>", label);
-		g_free (name);
-
-		return label;
+		return g_strdup (_("Image File"));
 	}
 
-	type = brasero_medium_get_type_string (BRASERO_MEDIUM (self));
+	if (media & BRASERO_MEDIUM_HAS_AUDIO) {
+		const gchar *audio_name;
+
+		audio_name = brasero_medium_get_CD_TEXT_title (BRASERO_MEDIUM (self));
+		if (audio_name)
+			return g_strdup (audio_name);
+	}
 
 	volume = brasero_volume_get_gvolume (self);
+	if (!volume)
+		goto last_chance;
+
 	name = g_volume_get_name (volume);
 	g_object_unref (volume);
 
-	if (name && name [0] != '\0') {
-		if (with_markup)
-			/* NOTE for translators: the first %s is the disc type and the
-			 * second %s the label of the already existing session on this disc. */
-			label = g_strdup_printf (_("<b>Data %s</b>: \"%s\""),
-						 type,
-						 name);
-		else
-			label = g_strdup_printf (_("Data %s: \"%s\""),
-						 type,
-						 name);
+	if (name)
+		return name;
 
-		g_free (name);
-		return label;
-	}
-	g_free (name);
-
-	drive = brasero_medium_get_drive (BRASERO_MEDIUM (self));
-	name = brasero_drive_get_display_name (drive);
+last_chance:
 
+	type = brasero_medium_get_type_string (BRASERO_MEDIUM (self));
+	name = NULL;
 	if (media & BRASERO_MEDIUM_BLANK) {
-		if (with_markup)
-			/* NOTE for translators: the first %s is the disc type and the
-			 * second %s the name of the drive this disc is in. */
-			label = g_strdup_printf (_("<b>Blank %s</b> in %s"),
-						 type,
-						 name);
-		else
-			label = g_strdup_printf (_("Blank %s in %s"),
-						 type,
-						 name);
+		/* NOTE for translators: the first %s is the disc type. */
+		name = g_strdup_printf (_("Blank %s"), type);
 	}
 	else if (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_HAS_AUDIO|BRASERO_MEDIUM_HAS_DATA)) {
-		if (with_markup)
-			/* NOTE for translators: the first %s is the disc type and the
-			 * second %s the name of the drive this disc is in. */
-			label = g_strdup_printf (_("<b>Audio and data %s</b> in %s"),
-						 type,
-						 name);
-		else
-			label = g_strdup_printf (_("Audio and data %s in %s"),
-						 type,
-						 name);
+		/* NOTE for translators: the first %s is the disc type. */
+		name = g_strdup_printf (_("Audio and data %s"), type);
 	}
 	else if (media & BRASERO_MEDIUM_HAS_AUDIO) {
-		if (with_markup)
-			/* NOTE for translators: the first %s is the disc type and the
-			 * second %s the name of the drive this disc is in. */
-			label = g_strdup_printf (_("<b>Audio %s</b> in %s"),
-						 type,
-						 name);
-		else
-			label = g_strdup_printf (_("Audio %s in %s"),
-						 type,
-						 name);
+		/* NOTE for translators: the first %s is the disc type. */
+		name = g_strdup_printf (_("Audio %s"), type);
 	}
 	else if (media & BRASERO_MEDIUM_HAS_DATA) {
-		if (with_markup)
-			/* NOTE for translators: the first %s is the disc type and the
-		 	* second %s the name of the drive this disc is in. */
-			label = g_strdup_printf (_("<b>Data %s</b> in %s"),
-						 type,
-						 name);
-		else
-			label = g_strdup_printf (_("Data %s in %s"),
-						 type,
-						 name);
+		/* NOTE for translators: the first %s is the disc type. */
+		name = g_strdup_printf (_("Data %s"), type);
 	}
 	else {
-		if (with_markup)
-			/* NOTE for translators: the first %s is the disc type and the
-		 	* second %s the name of the drive this disc is in. */
-			label = g_strdup_printf (_("<b>%s</b> in %s"),
-						 type,
-						 name);
-		else
-			label = g_strdup_printf (_("%s in %s"),
-						 type,
-						 name);
+		/* NOTE for translators: the first %s is the disc type. */
+		name = g_strdup (type);
 	}
 
-	g_free (name);
-	return label;
+	return name;
 }
 
 static void
