@@ -7,19 +7,21 @@
  ***************************************************************************/
 
 /*
- *  This program is free software; you can redistribute it and/or modify
+ *  Brasero is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Brasero is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -30,6 +32,10 @@
 #include <stdio.h>
 #include <string.h>
 
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+
 #include <glib.h>
 #include <glib-object.h>
 #include <glib/gi18n-lib.h>
@@ -57,6 +63,7 @@ struct _BraseroBurnPrivate {
 	BraseroBurnSession *session;
 
 	GMainLoop *sleep_loop;
+	guint timeout_id;
 
 	guint tasks_done;
 	guint task_nb;
@@ -67,6 +74,9 @@ struct _BraseroBurnPrivate {
 
 	gint appcookie;
 
+	guint64 session_start;
+	guint64 session_end;
+
 	guint src_locked:1;
 	guint dest_locked:1;
 
@@ -122,11 +132,11 @@ static guint brasero_burn_signals [LAST_SIGNAL] = { 0 };
 
 #define BRASERO_BURN_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), BRASERO_TYPE_BURN, BraseroBurnPrivate))
 
-#define MAX_EJECT_WAIT_TIME	20000
-#define MAX_MOUNT_ATTEMPTS	10
+#define MAX_EJECT_ATTEMPTS	5
+#define MAX_MOUNT_ATTEMPTS	20
 #define MOUNT_TIMEOUT		500
 
-#define MAX_LOAD_ATTEMPTS	20
+#define MAX_LOAD_ATTEMPTS	50
 #define LOAD_TIMEOUT		500
 
 static GObjectClass *parent_class = NULL;
@@ -162,9 +172,7 @@ brasero_burn_log (BraseroBurn *burn,
 	va_list arg_list;
 
 	va_start (arg_list, format);
-
 	brasero_burn_session_logv (priv->session, format, arg_list);
-
 	va_end (arg_list);
 }
 
@@ -200,6 +208,7 @@ brasero_burn_wakeup (BraseroBurn *burn)
 	if (priv->sleep_loop)
 		g_main_loop_quit (priv->sleep_loop);
 
+	priv->timeout_id = 0;
 	return FALSE;
 }
 
@@ -207,16 +216,24 @@ static BraseroBurnResult
 brasero_burn_sleep (BraseroBurn *burn, gint msec)
 {
 	BraseroBurnPrivate *priv = BRASERO_BURN_PRIVATE (burn);
+	GMainLoop *loop;
 
 	priv->sleep_loop = g_main_loop_new (NULL, FALSE);
-	g_timeout_add (msec,
-		       (GSourceFunc) brasero_burn_wakeup,
-		       burn);
+	priv->timeout_id = g_timeout_add (msec,
+					  (GSourceFunc) brasero_burn_wakeup,
+					  burn);
 
-	g_main_loop_run (priv->sleep_loop);
+	/* Keep a reference to the loop in case we are cancelled to destroy it */
+	loop = priv->sleep_loop;
+	g_main_loop_run (loop);
 
+	if (priv->timeout_id) {
+		g_source_remove (priv->timeout_id);
+		priv->timeout_id = 0;
+	}
+
+	g_main_loop_unref (loop);
 	if (priv->sleep_loop) {
-		g_main_loop_unref (priv->sleep_loop);
 		priv->sleep_loop = NULL;
 		return BRASERO_BURN_OK;
 	}
@@ -226,63 +243,64 @@ brasero_burn_sleep (BraseroBurn *burn, gint msec)
 }
 
 static BraseroBurnResult
-brasero_burn_wait_for_dest_insertion (BraseroBurn *burn,
-				      GError **error)
+brasero_burn_reprobe (BraseroBurn *burn)
 {
-	BraseroBurnPrivate *priv = BRASERO_BURN_PRIVATE (burn);
 	BraseroMedium *medium;
-	guint attempt = 0;
-	gchar *failure;
+	BraseroBurnPrivate *priv;
+	BraseroBurnResult result = BRASERO_BURN_OK;
 
-	BRASERO_BURN_LOG ("Waiting for destination disc");
-	if (!priv->dest)
-		return BRASERO_BURN_OK;
+	priv = BRASERO_BURN_PRIVATE (burn);
 
-	/* we need to release our lock */
-	if (priv->dest_locked) {
-		priv->dest_locked = 0;
-		if (!brasero_drive_unlock (priv->dest)) {
+	/* reprobe the medium and wait for it to be probed */
+	brasero_drive_reprobe (priv->dest);
+	while (!(medium = brasero_drive_get_medium (priv->dest)))
+		result = brasero_burn_sleep (burn, 250);
+
+	return result;
+}
+
+static BraseroBurnResult
+brasero_burn_eject (BraseroBurn *self,
+		    BraseroDrive *drive,
+		    GError **error)
+{
+	BraseroMedium *medium;
+	guint counter = 0;
+
+	medium = brasero_drive_get_medium (drive);
+	brasero_volume_eject (BRASERO_VOLUME (medium), TRUE, error);
+
+	/* sleep some time and see what happened */
+	brasero_burn_sleep (self, 500);
+	medium = brasero_drive_get_medium (drive);
+
+	/* Retry several times, since sometimes the drives are really busy */
+	while (medium && brasero_medium_get_status (medium) != BRASERO_MEDIUM_NONE) {
+		counter ++;
+		if (counter > MAX_EJECT_ATTEMPTS) {
 			gchar *name;
 
-			name = brasero_drive_get_display_name (priv->dest);
-			g_set_error (error,
-				     BRASERO_BURN_ERROR,
-				     BRASERO_BURN_ERROR_GENERAL,
-				     _("\"%s\" can't be unlocked"),
-				     name);
+			BRASERO_BURN_LOG ("Max attempts reached at ejecting");
+
+			/* FIXME: it'd be better if we asked the user to do it
+			 * manually */
+			name = brasero_drive_get_display_name (drive);
+			if (error && !(*error))
+				g_set_error (error,
+					     BRASERO_BURN_ERROR,
+					     BRASERO_BURN_ERROR_GENERAL,
+					     _("the media in %s can't be ejected"),
+					     name);
 			g_free (name);
 			return BRASERO_BURN_ERR;
 		}
-	}
 
-	medium = brasero_drive_get_medium (priv->dest);
-	while (brasero_medium_get_status (medium) == BRASERO_MEDIUM_NONE) {
-		brasero_burn_sleep (burn, LOAD_TIMEOUT);
-		
-		attempt ++;
-		if (attempt > MAX_LOAD_ATTEMPTS) {
-			g_set_error (error,
-				     BRASERO_BURN_ERROR,
-				     BRASERO_BURN_ERROR_GENERAL,
-				     _("the disc could not be reloaded (max attemps reached)"));
-			return BRASERO_BURN_ERR;
-		}
-
-		medium = brasero_drive_get_medium (priv->dest);
+		BRASERO_BURN_LOG ("Retrying ejection");
+		brasero_volume_eject (BRASERO_VOLUME (medium), TRUE, error);
+		brasero_burn_sleep (self, 500);
+		medium = brasero_drive_get_medium (drive);
 	}
 
-	/* Re-add the lock */
-	if (!priv->dest_locked
-	&&  !brasero_drive_lock (priv->dest, _("ongoing burning process"), &failure)) {
-		g_set_error (error,
-			     BRASERO_BURN_ERROR,
-			     BRASERO_BURN_ERROR_GENERAL,
-			     _("the drive can't be locked (%s)"),
-			     failure);
-		return BRASERO_BURN_ERR;
-	}
-	priv->dest_locked = 1;
-
 	return BRASERO_BURN_OK;
 }
 
@@ -291,11 +309,13 @@ brasero_burn_eject_dest_media (BraseroBurn *self,
 			       GError **error)
 {
 	BraseroBurnPrivate *priv;
+	BraseroBurnResult result;
 	BraseroMedium *medium;
 
 	priv = BRASERO_BURN_PRIVATE (self);
 
 	BRASERO_BURN_LOG ("Ejecting destination disc");
+
 	if (!priv->dest)
 		return BRASERO_BURN_OK;
 
@@ -319,22 +339,8 @@ brasero_burn_eject_dest_media (BraseroBurn *self,
 		}
 	}
 
-	if (!brasero_volume_eject (BRASERO_VOLUME (medium), TRUE, NULL)) {
-		gchar *name;
-
-		name = brasero_drive_get_display_name (priv->dest);
-
-		g_set_error (error,
-			     BRASERO_BURN_ERROR,
-			     BRASERO_BURN_ERROR_GENERAL,
-			     _("the media in %s can't be ejected"),
-			     name);
-
-		g_free (name);
-
-		priv->dest = NULL;
-		return BRASERO_BURN_ERR;
-	}
+	result = brasero_burn_eject (self, priv->dest, error);
+	priv->dest = NULL;
 
 	return BRASERO_BURN_OK;
 }
@@ -344,13 +350,17 @@ brasero_burn_eject_src_media (BraseroBurn *self,
 			      GError **error)
 {
 	BraseroBurnPrivate *priv;
+	BraseroBurnResult result;
 	BraseroMedium *medium;
 
 	priv = BRASERO_BURN_PRIVATE (self);
 
+	BRASERO_BURN_LOG ("Ejecting source disc");
+
 	if (!priv->src)
 		return BRASERO_BURN_OK;
 
+	/* Release lock, unmount, ... */
 	medium = brasero_drive_get_medium (priv->src);
 	if (brasero_volume_is_mounted (BRASERO_VOLUME (medium))) {
 		BraseroBurnResult result;
@@ -376,27 +386,11 @@ brasero_burn_eject_src_media (BraseroBurn *self,
 		}
 	}
 
-	brasero_volume_eject (BRASERO_VOLUME (medium), TRUE, NULL);
-	medium = brasero_drive_get_medium (priv->src);
-	if (medium && brasero_medium_get_status (medium) != BRASERO_MEDIUM_NONE) {
-		gchar *name;
-
-		name = brasero_drive_get_display_name (priv->src);
-
-		g_set_error (error,
-			     BRASERO_BURN_ERROR,
-			     BRASERO_BURN_ERROR_GENERAL,
-			     _("the media in %s can't be ejected"),
-			     name);
-
-		g_free (name);
-
-		priv->src = NULL;
-		return BRASERO_BURN_ERR;
-	}
-
+	/* and eject */
+	result = brasero_burn_eject (self, priv->src, error);
 	priv->src = NULL;
-	return BRASERO_BURN_OK;
+
+	return result;
 }
 
 static BraseroBurnResult
@@ -472,20 +466,9 @@ brasero_burn_ask_for_dest_media (BraseroBurn *burn,
 				 BraseroMedia required_media,
 				 GError **error)
 {
-	BraseroMedia media;
 	BraseroMedium *medium;
 	BraseroBurnPrivate *priv = BRASERO_BURN_PRIVATE (burn);
 
-	medium = brasero_drive_get_medium (priv->dest);
-	media = brasero_medium_get_status (medium);
-	if (media != BRASERO_MEDIUM_NONE) {
-		BraseroBurnResult result;
-
-		result = brasero_burn_eject_dest_media (burn, error);
-		if (result != BRASERO_BURN_OK)
-			return result;
-	}
-
 	if (!priv->dest) {
 		priv->dest = brasero_burn_session_get_burner (priv->session);
 		if (!priv->dest) {
@@ -497,6 +480,15 @@ brasero_burn_ask_for_dest_media (BraseroBurn *burn,
 		}
 	}
 
+	medium = brasero_drive_get_medium (priv->dest);
+	if (medium || brasero_medium_get_status (medium) != BRASERO_MEDIUM_NONE) {
+		BraseroBurnResult result;
+
+		result = brasero_burn_eject_dest_media (burn, error);
+		if (result != BRASERO_BURN_OK)
+			return result;
+	}
+
 	return brasero_burn_ask_for_media (burn,
 					   priv->dest,
 					   error_type,
@@ -708,6 +700,7 @@ brasero_burn_is_loaded_dest_media_supported (BraseroBurn *burn,
 			*must_blank = FALSE;
 		else
 			*must_blank = TRUE;
+
 		return BRASERO_BURN_ERROR_NONE;
 	}
 
@@ -732,14 +725,16 @@ brasero_burn_is_loaded_dest_media_supported (BraseroBurn *burn,
 
 		return BRASERO_BURN_ERROR_MEDIA_NOT_WRITABLE;
 	}
-	else if (unsupported & (BRASERO_MEDIUM_DVD|BRASERO_MEDIUM_DVD_DL))
+	else if (unsupported & BRASERO_MEDIUM_DVD)
 		return BRASERO_BURN_ERROR_DVD_NOT_SUPPORTED;
 
 	return BRASERO_BURN_ERROR_MEDIA_UNSUPPORTED;
 }
 
 static BraseroBurnResult
-brasero_burn_lock_dest_media (BraseroBurn *burn, GError **error)
+brasero_burn_lock_dest_media (BraseroBurn *burn,
+			      BraseroBurnError *ret_error,
+			      GError **error)
 {
 	gchar *failure;
 	BraseroMedia media;
@@ -765,8 +760,6 @@ brasero_burn_lock_dest_media (BraseroBurn *burn, GError **error)
 
 	result = BRASERO_BURN_OK;
 
-again:
-
 	medium = brasero_drive_get_medium (priv->dest);
 	if (!medium) {
 		result = BRASERO_BURN_NEED_RELOAD;
@@ -795,7 +788,7 @@ again:
 	BRASERO_BURN_LOG_WITH_FULL_TYPE (BRASERO_TRACK_TYPE_DISC,
 					 media,
 					 BRASERO_PLUGIN_IO_NONE,
-					 "Waiting for dest drive");
+					 "Media inserted is");
 
 	if (priv->dest_locked) {
 		/* NOTE: after a blanking, for nautilus_burn the CD/DVD is still
@@ -827,9 +820,8 @@ again:
 	berror = brasero_burn_is_loaded_dest_media_supported (burn,
 							      media,
 							      &must_blank);
-
 	if (berror != BRASERO_BURN_ERROR_NONE) {
-		BRASERO_BURN_LOG ("the media is not supported");
+		BRASERO_BURN_LOG ("Inserted media is not supported");
 		result = BRASERO_BURN_NEED_RELOAD;
 		goto end;
 	}
@@ -878,14 +870,15 @@ again:
 		/* NOTE: no need to error out here since the only thing
 		 * we are interested in is if it is AUDIO or not or if
 		 * the disc we are copying has audio tracks only or not */
-		if (input.type == BRASERO_TRACK_TYPE_AUDIO) {
+		if (input.type == BRASERO_TRACK_TYPE_AUDIO
+		&& !(input.subtype.audio_format & (BRASERO_VIDEO_FORMAT_UNDEFINED|
+						   BRASERO_VIDEO_FORMAT_VCD|
+						   BRASERO_VIDEO_FORMAT_VIDEO_DVD))) {
 			result = brasero_burn_emit_signal (burn, WARN_REWRITABLE_SIGNAL, BRASERO_BURN_CANCEL);
 			if (result != BRASERO_BURN_OK)
 				goto end;
 		}
 
-		/* FIXME: if NO_TMP_FILE is not set then the warning won't get
-		 * emitted */
 		if (input.type == BRASERO_TRACK_TYPE_DISC
 		&& (input.subtype.media & (BRASERO_MEDIUM_HAS_AUDIO|BRASERO_MEDIUM_HAS_DATA)) == BRASERO_MEDIUM_HAS_AUDIO) {
 			result = brasero_burn_emit_signal (burn, WARN_REWRITABLE_SIGNAL, BRASERO_BURN_CANCEL);
@@ -908,25 +901,14 @@ again:
 
 end:
 
-	if (result == BRASERO_BURN_NEED_RELOAD) {
-		BraseroMedia required_media;
-
-		required_media = brasero_burn_caps_get_required_media_type (priv->caps,
-									    priv->session);
-
-		result = brasero_burn_ask_for_dest_media (burn,
-							  berror,
-							  required_media,
-							  error);
-		if (result == BRASERO_BURN_OK)
-			goto again;
-	}
-
-	if (result != BRASERO_BURN_OK) {
+	if (result != BRASERO_BURN_OK && priv->dest_locked) {
 		priv->dest_locked = 0;
 		brasero_drive_unlock (priv->dest);
 	}
 
+	if (result == BRASERO_BURN_ERROR_RELOAD_MEDIA && ret_error)
+		*ret_error = berror;
+
 	return result;
 }
 
@@ -943,7 +925,12 @@ again:
 
 	/* eject and ask the user to reload a disc */
 	required_media = brasero_burn_caps_get_required_media_type (priv->caps, priv->session);
-	required_media &= (BRASERO_MEDIUM_WRITABLE|BRASERO_MEDIUM_CD|BRASERO_MEDIUM_DVD|BRASERO_MEDIUM_DVD_DL);
+	required_media &= (BRASERO_MEDIUM_WRITABLE|
+			   BRASERO_MEDIUM_CD|
+			   BRASERO_MEDIUM_DVD);
+
+	if (required_media == BRASERO_MEDIUM_NONE)
+		required_media = BRASERO_MEDIUM_WRITABLE;
 
 	result = brasero_burn_ask_for_dest_media (burn,
 						  error_code,
@@ -952,7 +939,9 @@ again:
 	if (result != BRASERO_BURN_OK)
 		return result;
 
-	result = brasero_burn_lock_dest_media (burn, error);
+	result = brasero_burn_lock_dest_media (burn,
+					       &error_code,
+					       error);
 	if (result == BRASERO_BURN_NEED_RELOAD)
 		goto again;
 
@@ -985,6 +974,8 @@ brasero_burn_mount_media (BraseroBurn *self,
 			return BRASERO_BURN_ERR;
 		}
 
+		BRASERO_BURN_LOG ("Trying to mount medium");
+
 		/* NOTE: we don't really care about the return value */
 		brasero_volume_mount (BRASERO_VOLUME (medium), FALSE, NULL);
 		priv->mounted_by_us = TRUE;
@@ -1065,7 +1056,8 @@ again:
 }
 
 static BraseroBurnResult
-brasero_burn_unlock_src_media (BraseroBurn *burn)
+brasero_burn_unlock_src_media (BraseroBurn *burn,
+			       GError **error)
 {
 	BraseroBurnPrivate *priv = BRASERO_BURN_PRIVATE (burn);
 	BraseroMedium *medium;
@@ -1080,7 +1072,7 @@ brasero_burn_unlock_src_media (BraseroBurn *burn)
 
 	medium = brasero_drive_get_medium (priv->src);
 	if (priv->mounted_by_us) {
-		brasero_volume_umount (BRASERO_VOLUME (medium), TRUE, NULL);
+		brasero_volume_umount (BRASERO_VOLUME (medium), TRUE, error);
 		priv->mounted_by_us = 0;
 	}
 
@@ -1088,14 +1080,15 @@ brasero_burn_unlock_src_media (BraseroBurn *burn)
 	brasero_drive_unlock (priv->src);
 
 	if (BRASERO_BURN_SESSION_EJECT (priv->session))
-		brasero_volume_eject (BRASERO_VOLUME (medium), FALSE, NULL);
+		brasero_volume_eject (BRASERO_VOLUME (medium), FALSE, error);
 
 	priv->src = NULL;
 	return BRASERO_BURN_OK;
 }
 
 static BraseroBurnResult
-brasero_burn_unlock_dest_media (BraseroBurn *burn)
+brasero_burn_unlock_dest_media (BraseroBurn *burn,
+				GError **error)
 {
 	BraseroBurnPrivate *priv = BRASERO_BURN_PRIVATE (burn);
 	BraseroMedium *medium;
@@ -1113,17 +1106,18 @@ brasero_burn_unlock_dest_media (BraseroBurn *burn)
 	medium = brasero_drive_get_medium (priv->dest);
 
 	if (BRASERO_BURN_SESSION_EJECT (priv->session))
-		brasero_volume_eject (BRASERO_VOLUME (medium), FALSE, NULL);
+		brasero_volume_eject (BRASERO_VOLUME (medium), FALSE, error);
 
 	priv->dest = NULL;
 	return BRASERO_BURN_OK;
 }
 
 static BraseroBurnResult
-brasero_burn_unlock_medias (BraseroBurn *burn)
+brasero_burn_unlock_medias (BraseroBurn *burn,
+			    GError **error)
 {
-	brasero_burn_unlock_dest_media (burn);
-	brasero_burn_unlock_src_media (burn);
+	brasero_burn_unlock_dest_media (burn, error);
+	brasero_burn_unlock_src_media (burn, error);
 
 	return BRASERO_BURN_OK;
 }
@@ -1172,7 +1166,40 @@ brasero_burn_action_changed (BraseroTask *task,
 			     BraseroBurnAction action,
 			     BraseroBurn *burn)
 {
+	BraseroBurnPrivate *priv;
+	BraseroMedia media;
+
 	brasero_burn_action_changed_real (burn, action);
+
+	if (action != BRASERO_BURN_ACTION_START_RECORDING)
+		return;
+
+	priv = BRASERO_BURN_PRIVATE (burn);
+
+	media = brasero_burn_session_get_dest_media (priv->session);
+	if ((BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_PLUS)
+	||   BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_RESTRICTED))) {
+		BraseroBurnFlag flags;
+		BraseroMedium *medium;
+		BraseroDrive *drive;
+		gint64 len = 0;
+
+		drive = brasero_burn_session_get_burner (priv->session);
+		medium = brasero_drive_get_medium (drive);
+		flags = brasero_burn_session_get_flags (priv->session);
+
+		/* we need to save some parameters for later checksuming */
+		brasero_task_ctx_get_session_output_size (BRASERO_TASK_CTX (priv->task),
+							  &len,
+							  NULL);
+
+		if (flags & (BRASERO_BURN_FLAG_MERGE|BRASERO_BURN_FLAG_APPEND))
+			priv->session_start = brasero_medium_get_next_writable_address (medium);
+		else
+			priv->session_start = 0;
+
+		priv->session_end = priv->session_start + len;
+	}
 }
 
 void
@@ -1206,15 +1233,21 @@ brasero_burn_status (BraseroBurn *burn,
 	g_return_val_if_fail (BRASERO_BURN (burn), BRASERO_BURN_ERR);
 	
 	priv = BRASERO_BURN_PRIVATE (burn);
-	if (!priv->task || !brasero_task_is_running (priv->task))
+
+	if (!priv->task)
+		return BRASERO_BURN_NOT_READY;
+
+	if (isosize)
+		brasero_task_ctx_get_session_output_size (BRASERO_TASK_CTX (priv->task),
+							  NULL,
+							  isosize);
+
+	if (!brasero_task_is_running (priv->task))
 		return BRASERO_BURN_NOT_READY;
 
 	if (rate)
 		brasero_task_ctx_get_rate (BRASERO_TASK_CTX (priv->task), rate);
 
-	if (isosize)
-		brasero_task_ctx_get_session_output_size (BRASERO_TASK_CTX (priv->task), NULL, isosize);
-
 	if (written)
 		brasero_task_ctx_get_written (BRASERO_TASK_CTX (priv->task), written);
 
@@ -1390,6 +1423,25 @@ start:
 	return BRASERO_BURN_ERR;
 }
 
+static BraseroBurnResult
+brasero_burn_can_use_drive_exclusively (BraseroBurn *burn,
+					BraseroDrive *drive)
+{
+	BraseroBurnResult result;
+
+	if (!drive)
+		return BRASERO_BURN_OK;
+
+	while (!brasero_drive_can_use_exclusively (drive)) {
+		BRASERO_BURN_LOG ("Device busy, retrying in 250 ms");
+		result = brasero_burn_sleep (burn, 250);
+		if (result == BRASERO_BURN_CANCEL)
+			return result;
+	}
+
+	return BRASERO_BURN_OK;
+}
+
 static BraseroBurnResult
 brasero_burn_run_recorder (BraseroBurn *burn, GError **error)
 {
@@ -1404,11 +1456,23 @@ brasero_burn_run_recorder (BraseroBurn *burn, GError **error)
 	BraseroBurnPrivate *priv = BRASERO_BURN_PRIVATE (burn);
 
 	has_slept = FALSE;
+
 	src = brasero_burn_session_get_src_drive (priv->session);
 	src_medium = brasero_drive_get_medium (src);
+
 	burner = brasero_burn_session_get_burner (priv->session);
 	burnt_medium = brasero_drive_get_medium (burner);
 
+	/* before we start let's see if that drive can be used exclusively.
+	 * Of course, it's not really safe since a process could take a lock
+	 * just after us but at least it'll give some time to HAL and friends
+	 * to finish what they're doing. 
+	 * This was done because more than often backends wouldn't be able to 
+	 * get a lock on a medium after a simulation. */
+	result = brasero_burn_can_use_drive_exclusively (burn, burner);
+	if (result != BRASERO_BURN_OK)
+		return result;
+
 start:
 
 	/* this is just in case */
@@ -1635,10 +1699,15 @@ brasero_burn_run_tasks (BraseroBurn *burn,
 			priv->task = NULL;
 			priv->tasks_done ++;
 
-			/* Now it can happen (like with dvd+rw-format) that for
-			 * the whole OS, the disc doesn't exist during the 
+			/* Reprobe. It can happen (like with dvd+rw-format) that
+			 * for the whole OS, the disc doesn't exist during the 
 			 * formatting. Wait for the disc to reappear */
-			result = brasero_burn_wait_for_dest_insertion (burn, error);
+			/*  Likewise, this is necessary when we do a
+			 * simulation before blanking since it blanked the disc
+			 * and then to create all tasks necessary for the real
+			 * burning operation, we'll need the real medium status 
+			 * not to include a blanking job again. */
+			result = brasero_burn_reprobe (burn);
 			if (result != BRASERO_BURN_OK)
 				break;
 
@@ -1776,17 +1845,19 @@ static BraseroBurnResult
 brasero_burn_check_session_consistency (BraseroBurn *burn,
 					GError **error)
 {
+	BraseroMedia media;
 	BraseroBurnFlag flag;
 	BraseroTrackType type;
 	BraseroBurnFlag flags;
 	BraseroBurnFlag retval;
+	BraseroBurnResult result;
 	BraseroBurnFlag supported = BRASERO_BURN_FLAG_NONE;
 	BraseroBurnFlag compulsory = BRASERO_BURN_FLAG_NONE;
 	BraseroBurnPrivate *priv = BRASERO_BURN_PRIVATE (burn);
 
 	BRASERO_BURN_DEBUG (burn, "Checking session consistency");
 
-	/* make sure there is a session, a burner */
+	/* make sure there is a track in the session. */
 	brasero_burn_session_get_input_type (priv->session, &type);
 	if (type.type == BRASERO_TRACK_TYPE_NONE
 	|| !brasero_burn_session_get_tracks (priv->session)) {
@@ -1798,7 +1869,7 @@ brasero_burn_check_session_consistency (BraseroBurn *burn,
 		return BRASERO_BURN_ERR;
 	}
 
-	/* make sure there is a drive set as burner */
+	/* make sure there is a drive set as burner. */
 	if (!brasero_burn_session_is_dest_file (priv->session)) {
 		BraseroDrive *burner;
 
@@ -1818,79 +1889,69 @@ brasero_burn_check_session_consistency (BraseroBurn *burn,
 			brasero_burn_session_set_num_copies (priv->session, 1);
 	}
 
+	media = brasero_burn_session_get_dest_media (priv->session);
+
 	/* save then wipe out flags from session to check them one by one */
 	flags = brasero_burn_session_get_flags (priv->session);
 	brasero_burn_session_remove_flag (priv->session, flags);
 
-	brasero_burn_caps_get_flags (priv->caps,
-				     priv->session,
-				     &supported,
-				     &compulsory);
+	result = brasero_burn_caps_get_flags (priv->caps,
+					      priv->session,
+					      &supported,
+					      &compulsory);
+	if (result != BRASERO_BURN_OK)
+		return result;
 
 	for (flag = 1; flag < BRASERO_BURN_FLAG_LAST; flag <<= 1) {
-		/* check each flag before re-adding it */
-		if ((flags & flag) && (supported & flag)) {
+		/* see if this flag was originally set */
+		if (!(flags & flag))
+			continue;
+
+		/* Check each flag before re-adding it. Emit warnings to user
+		 * to know if he wants to carry on for some flags when they are
+		 * not supported; namely DUMMY. Other flags trigger an error.
+		 * No need for BURNPROOF since that usually means it is just the
+		 * media type that doesn't need it. */
+		if (supported & flag) {
 			brasero_burn_session_add_flag (priv->session, flag);
 			brasero_burn_caps_get_flags (priv->caps,
 						     priv->session,
 						     &supported,
 						     &compulsory);
 		}
-		else if (flags & flag)
-			BRASERO_BURN_LOG_FLAGS (flag, "Flag set but not supported");
+		else {
+			BRASERO_BURN_LOG_FLAGS (flag, "Flag set but not supported:");
+
+			if (flag & BRASERO_BURN_FLAG_DUMMY) {
+				/* This is simply a warning that it's not possible */
+
+			}
+			else if (flag & BRASERO_BURN_FLAG_MERGE) {
+				/* we pay attention to one flag in particular
+				 * (MERGE) if it was set then it must be
+				 * supported. Otherwise error out. */
+				g_set_error (error,
+					     BRASERO_BURN_ERROR,
+					     BRASERO_BURN_ERROR_GENERAL,
+					     _("merging data is impossible with this disc"));
+				return BRASERO_BURN_ERR;
+			}
+			else if ((flag & BRASERO_BURN_FLAG_BURNPROOF)
+			     && !(BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_PLUS)
+			     ||   BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_RESTRICTED))) {
+				 /* Warn the user BURNPROOF won't be possible */
+
+			}
+		}
 	}
 
 	retval = brasero_burn_session_get_flags (priv->session);
-
-	if ((flags & BRASERO_BURN_FLAG_MERGE)
-	&& !(retval & BRASERO_BURN_FLAG_MERGE)) {
-		/* we pay attention to one flag in particular (MERGE) if it was
-		 * set then it must be supported. Otherwise error out. */
-		g_set_error (error,
-			     BRASERO_BURN_ERROR,
-			     BRASERO_BURN_ERROR_GENERAL,
-			     _("merging data is impossible with this disc"));
-		return BRASERO_BURN_ERR;
-	}
-
 	if (retval != flags)
-		BRASERO_BURN_LOG_FLAGS (flags, "Some flags were not supported. Corrected to ");
+		BRASERO_BURN_LOG_FLAGS (retval, "Some flags were not supported. Corrected to");
 
 	if (retval != (retval | compulsory)) {
-		BRASERO_BURN_DEBUG (burn,
-				    "Some compulsory flags were forgotten. Corrected to ",
-				   (retval & compulsory),
-				    compulsory);
-
-		brasero_burn_session_add_flag (priv->session, compulsory);
-	}
-
-	/* we check flags consistency 
-	 * NOTE: should we return an error if they are not consistent? */
-	if ((type.type != BRASERO_TRACK_TYPE_AUDIO
-	&&   type.type != BRASERO_TRACK_TYPE_DATA
-	&&   type.type != BRASERO_TRACK_TYPE_DISC)
-	||   brasero_burn_session_is_dest_file (priv->session)) {
-		if (retval & BRASERO_BURN_FLAG_MERGE) {
-			BRASERO_BURN_DEBUG (burn, "Inconsistent flag: you can't use flag merge");
-			retval &= ~BRASERO_BURN_FLAG_MERGE;
-		}
-			
-		if (retval & BRASERO_BURN_FLAG_APPEND) {
-			BRASERO_BURN_DEBUG (burn, "Inconsistent flags: you can't use flag append");
-			retval &= ~BRASERO_BURN_FLAG_APPEND;
-		}
-
-		if (retval & BRASERO_BURN_FLAG_NO_TMP_FILES) {
-			BRASERO_BURN_DEBUG (burn, "Inconsistent flag: you can't use flag on_the_fly");
-			retval &= ~BRASERO_BURN_FLAG_NO_TMP_FILES;
-		}
-	}
-
-	if (brasero_burn_session_is_dest_file (priv->session)
-	&& (retval & BRASERO_BURN_FLAG_DONT_CLEAN_OUTPUT) == 0) {
-		BRASERO_BURN_DEBUG (burn, "Forgotten flag: you must use flag dont_clean_output");
-		retval |= BRASERO_BURN_FLAG_DONT_CLEAN_OUTPUT;
+		retval |= compulsory;
+		BRASERO_BURN_LOG_FLAGS (retval, "Some compulsory flags were forgotten. Corrected to");
 	}
 
 	brasero_burn_session_set_flags (priv->session, retval);
@@ -2022,22 +2083,6 @@ brasero_burn_record_session (BraseroBurn *burn,
 	if (type == BRASERO_CHECKSUM_NONE)
 		return BRASERO_BURN_OK;
 
-	/* unlock dest drive that's necessary if we want to check burnt medias
-	 * it seems that the kernel caches its contents and can't/don't update
-	 * its caches after a blanking/recording. */
-	/* NOTE: that work if the disc had not been mounted before. That's the 
-	 * mount that triggers the caches. So maybe if the disc was blank (and
-	 * therefore couldn't have been previously mounted) we could skip that
-	 * unlock/eject step. A better way would be to have a system call to 
-	 * force a re-load. */
-
-/*	result = brasero_burn_eject_dest_media (burn, error);
-	if (result != BRASERO_BURN_OK)
-		return result;
-
-	priv->dest = NULL;
-*/
-
 	if (type == BRASERO_CHECKSUM_MD5
 	||  type == BRASERO_CHECKSUM_SHA1
 	||  type == BRASERO_CHECKSUM_SHA256) {
@@ -2079,34 +2124,59 @@ brasero_burn_record_session (BraseroBurn *burn,
 	 * anymore. BraseroBurnSession refs it. */
 	brasero_track_unref (track);
 
-	/* reload media */
-/*	result = brasero_burn_lock_checksum_media (burn, error);
-	if (result != BRASERO_BURN_OK)
-		return result;
-*/
-	/* this may be necessary for the drive to settle down */
-	/* and possibly be mounted by gnome-volume-manager (just temporarily) */
+	/* this may be necessary for the drive to settle down and possibly be
+	 * mounted by gnome-volume-manager (just temporarily) */
 	result = brasero_burn_sleep (burn, 5000);
 	if (result != BRASERO_BURN_OK)
 		return result;
 
+	/* reprobe the medium and wait for it to be probed */
+	result = brasero_burn_reprobe (burn);
+	if (result != BRASERO_BURN_OK)
+		return result;
+
 	medium = brasero_drive_get_medium (priv->dest);
-	brasero_medium_reload_info (medium);
 
 	if (type == BRASERO_CHECKSUM_MD5
 	||  type == BRASERO_CHECKSUM_SHA1
 	||  type == BRASERO_CHECKSUM_SHA256) {
-		guint track_num;
-		BraseroDrive *drive;
-		BraseroMedium *medium;
+		BraseroMedia media;
 
-		/* get the last track number */
-		drive = brasero_burn_session_get_burner (priv->session);
-		medium = brasero_drive_get_medium (drive);
-		track_num = brasero_medium_get_track_num (medium);
+		/* get the last written track address in case of DVD+RW/DVD-RW
+		 * restricted overwrite since there is no such thing as track
+		 * number for these drives. */
+		media = brasero_medium_get_status (medium);
 
-		BRASERO_BURN_LOG ("Last written track num == %i", track_num);
-		brasero_track_set_drive_track (track, track_num);
+		if (!BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_PLUS)
+		&&  !BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_RESTRICTED)) {
+			guint track_num;
+
+			track_num = brasero_medium_get_track_num (medium);
+
+			BRASERO_BURN_LOG ("Last written track num == %i", track_num);
+			brasero_track_set_drive_track (track, track_num);
+		}
+		else {
+			GValue *value;
+
+			value = g_new0 (GValue, 1);
+			g_value_init (value, G_TYPE_UINT64);
+
+			BRASERO_BURN_LOG ("Start of last written track address == %lli", priv->session_start);
+			g_value_set_uint64 (value, priv->session_start);
+			brasero_track_tag_add (track,
+					       BRASERO_TRACK_MEDIUM_ADDRESS_START_TAG,
+					       value);
+
+			value = g_new0 (GValue, 1);
+			g_value_init (value, G_TYPE_UINT64);
+
+			BRASERO_BURN_LOG ("End of last written track address == %lli", priv->session_end);
+			g_value_set_uint64 (value, priv->session_end);
+			brasero_track_tag_add (track,
+					       BRASERO_TRACK_MEDIUM_ADDRESS_END_TAG,
+					       value);
+		}
 	}
 
 	result = brasero_burn_check_real (burn, track, error);
@@ -2166,11 +2236,16 @@ brasero_burn_check (BraseroBurn *self,
 
 	result = brasero_burn_check_real (self, track, error);
 
-	brasero_burn_unlock_medias (self);
-
 	brasero_burn_powermanagement (self, FALSE);
 
+	if (result == BRASERO_BURN_OK)
+		result = brasero_burn_unlock_medias (self, error);
+	else
+		brasero_burn_unlock_medias (self, NULL);
+
 	/* no need to check the result of the comparison, it's set in session */
+
+	/* NOTE: unref session only AFTER drives are unlocked */
 	priv->session = NULL;
 	g_object_unref (session);
 
@@ -2178,8 +2253,8 @@ brasero_burn_check (BraseroBurn *self,
 }
 
 static BraseroBurnResult
-brasero_burn_same_src_dest (BraseroBurn *self,
-			    GError **error)
+brasero_burn_same_src_dest_image (BraseroBurn *self,
+				  GError **error)
 {
 	gchar *toc = NULL;
 	gchar *image = NULL;
@@ -2243,7 +2318,7 @@ brasero_burn_same_src_dest (BraseroBurn *self,
 	/* run */
 	result = brasero_burn_record_session (self, TRUE, error);
 	if (result != BRASERO_BURN_OK) {
-		brasero_burn_unlock_src_media (self);
+		brasero_burn_unlock_src_media (self, NULL);
 		goto end;
 	}
 
@@ -2269,6 +2344,81 @@ end:
 	return result;
 }
 
+static BraseroBurnResult
+brasero_burn_same_src_dest_reload_medium (BraseroBurn *burn,
+					  GError **error)
+{
+	BraseroBurnError berror;
+	BraseroBurnPrivate *priv;
+	BraseroBurnResult result;
+	BraseroMedia required_media;
+	BraseroBurnFlag session_flags;
+
+	priv = BRASERO_BURN_PRIVATE (burn);
+
+	BRASERO_BURN_LOG ("Reloading medium after copy");
+
+	/* Now there is the problem of flags... This really is a special
+	 * case. we need to try to adjust the flags to the media type
+	 * just after a new one is detected. For example there could be
+	 * BURNPROOF/DUMMY whereas they are not supported for DVD+RW. So
+	 * we need to be lenient. */
+
+	/* Eject and ask the user to reload a disc */
+	required_media = brasero_burn_caps_get_required_media_type (priv->caps,
+								    priv->session);
+	required_media &= (BRASERO_MEDIUM_WRITABLE|
+			   BRASERO_MEDIUM_CD|
+			   BRASERO_MEDIUM_DVD|
+			   BRASERO_MEDIUM_BD);
+
+	/* There is sometimes no way to determine which type of media is
+	 * required since some flags (that will be adjusted afterwards)
+	 * prevent to reach some media type like BURNPROOF and DVD+RW. */
+	if (required_media == BRASERO_MEDIUM_NONE)
+		required_media = BRASERO_MEDIUM_WRITABLE;
+
+	/* save the flags in case we modify them */
+	session_flags = brasero_burn_session_get_flags (priv->session);
+	berror = BRASERO_BURN_WARNING_INSERT_AFTER_COPY;
+
+again:
+
+	result = brasero_burn_ask_for_dest_media (burn,
+						  berror,
+						  required_media,
+						  error);
+	if (result != BRASERO_BURN_OK)
+		return result;
+
+	/* update the flags now before locking it since in lock function
+	 * we check the adequacy of the medium inserted. */
+	result = brasero_burn_check_session_consistency (burn, error);
+	if (result == BRASERO_BURN_CANCEL)
+		return result;
+
+	if (result != BRASERO_BURN_OK) {
+		/* Tell the user his/her disc is not supported and reload */
+		berror = BRASERO_BURN_ERROR_MEDIA_UNSUPPORTED;
+		brasero_burn_session_set_flags (priv->session, session_flags);
+		goto again;
+	}
+
+	/* One thing could make us fail now that flags and media type are
+	 * supported: the size. */
+	result = brasero_burn_lock_dest_media (burn, &berror, error);
+	if (result == BRASERO_BURN_CANCEL)
+		return result;
+
+	if (result != BRASERO_BURN_OK) {
+		/* Tell the user his/her disc is not supported and reload */
+		brasero_burn_session_set_flags (priv->session, session_flags);
+		goto again;
+	}
+
+	return BRASERO_BURN_OK;
+}
+
 BraseroBurnResult 
 brasero_burn_record (BraseroBurn *burn,
 		     BraseroBurnSession *session,
@@ -2292,22 +2442,39 @@ brasero_burn_record (BraseroBurn *burn,
 
 	if (brasero_burn_session_same_src_dest_drive (session)) {
 		/* This is a special case */
-		result = brasero_burn_same_src_dest (burn, error);
+		result = brasero_burn_same_src_dest_image (burn, error);
 		if (result != BRASERO_BURN_OK)
 			goto end;
 
-		/* lock the dest drive do it this way for the message */
-		result = brasero_burn_reload_dest_media (burn,
-							 BRASERO_BURN_WARNING_INSERT_AFTER_COPY,
-							 error);
+		result = brasero_burn_same_src_dest_reload_medium (burn, error);
 		if (result != BRASERO_BURN_OK)
 			goto end;
 	}
 	else if (!brasero_burn_session_is_dest_file (session)) {
+		BraseroBurnError berror;
+
 		/* do some drive locking quite early to make sure we have a
 		 * media in the drive so that we'll have all the necessary
 		 * information */
-		result = brasero_burn_lock_dest_media (burn, error);
+		result = brasero_burn_lock_dest_media (burn, &berror, error);
+		while (result == BRASERO_BURN_NEED_RELOAD) {
+			BraseroMedia required_media;
+
+			required_media = brasero_burn_caps_get_required_media_type (priv->caps,
+										    priv->session);
+			if (required_media == BRASERO_MEDIUM_NONE)
+				required_media = BRASERO_MEDIUM_WRITABLE;
+
+			result = brasero_burn_ask_for_dest_media (burn,
+								  berror,
+								  required_media,
+								  error);
+			if (result == BRASERO_BURN_OK)
+				goto end;
+
+			result = brasero_burn_lock_dest_media (burn, &berror, error);
+		}
+
 		if (result != BRASERO_BURN_OK)
 			goto end;
 	}
@@ -2318,7 +2485,8 @@ brasero_burn_record (BraseroBurn *burn,
 			goto end;
 	}
 
-	/* burn the session a first time whatever the number of copies required except if dummy session */
+	/* burn the session a first time whatever the number of copies required 
+	 * except if dummy session */
 	result = brasero_burn_record_session (burn, TRUE, error);
 	if (result == BRASERO_BURN_OK) {
 		gint num_copies;
@@ -2338,7 +2506,7 @@ brasero_burn_record (BraseroBurn *burn,
 
 			/* see if we still need it to be locked */
 			if (brasero_burn_session_get_input_type (session, NULL) != BRASERO_TRACK_TYPE_DISC)
-				brasero_burn_unlock_src_media (burn);
+				brasero_burn_unlock_src_media (burn, NULL);
 
 			result = brasero_burn_record_session (burn, TRUE, error);
 			if (result != BRASERO_BURN_OK)
@@ -2348,7 +2516,10 @@ brasero_burn_record (BraseroBurn *burn,
 
 end:
 
-	brasero_burn_unlock_medias (burn);
+	if (result == BRASERO_BURN_OK)
+		result = brasero_burn_unlock_medias (burn, error);
+	else
+		brasero_burn_unlock_medias (burn, NULL);
 
 	if (error && (*error) == NULL
 	&& (result == BRASERO_BURN_NOT_READY
@@ -2468,12 +2639,14 @@ end:
 	if (ret_error)
 		g_propagate_error (error, ret_error);
 
-	brasero_burn_unlock_medias (burn);
+	if (result == BRASERO_BURN_OK && !ret_error)
+		result = brasero_burn_unlock_medias (burn, error);
+	else
+		brasero_burn_unlock_medias (burn, NULL);
 
 	if (result == BRASERO_BURN_OK)
 		brasero_burn_action_changed_real (burn, BRASERO_BURN_ACTION_FINISHED);
 
-
 	brasero_burn_powermanagement (burn, FALSE);
 
 	/* release session */
@@ -2493,6 +2666,11 @@ brasero_burn_cancel (BraseroBurn *burn, gboolean protect)
 
 	priv = BRASERO_BURN_PRIVATE (burn);
 
+	if (priv->timeout_id) {
+		g_source_remove (priv->timeout_id);
+		priv->timeout_id = 0;
+	}
+
 	if (priv->sleep_loop) {
 		g_main_loop_quit (priv->sleep_loop);
 		priv->sleep_loop = NULL;
@@ -2509,6 +2687,11 @@ brasero_burn_finalize (GObject *object)
 {
 	BraseroBurnPrivate *priv = BRASERO_BURN_PRIVATE (object);
 
+	if (priv->timeout_id) {
+		g_source_remove (priv->timeout_id);
+		priv->timeout_id = 0;
+	}
+
 	if (priv->sleep_loop) {
 		g_main_loop_quit (priv->sleep_loop);
 		priv->sleep_loop = NULL;
