@@ -7,19 +7,21 @@
  ***************************************************************************/
 
 /*
- *  This program is free software; you can redistribute it and/or modify
+ *  Brasero is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Brasero is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -37,17 +39,13 @@
 #include <glib/gstdio.h>
 #include <gmodule.h>
 
-#include <libgnomevfs/gnome-vfs-utils.h>
-
-#include <nautilus-burn-drive.h>
-
 #include "burn-cdrdao-common.h"
 #include "burn-cdrdao.h"
 #include "burn-basics.h"
 #include "burn-plugin.h"
 #include "burn-job.h"
 #include "burn-process.h"
-#include "brasero-ncb.h"
+#include "burn-drive.h"
 #include "burn-medium.h"
 
 BRASERO_PLUGIN_BOILERPLATE (BraseroCdrdao, brasero_cdrdao, BRASERO_TYPE_PROCESS, BraseroProcess);
@@ -72,7 +70,7 @@ brasero_cdrdao_read_stderr_image (BraseroCdrdao *cdrdao, const gchar *line)
 		brasero_job_get_action (BRASERO_JOB (cdrdao), &action);
 		if (action == BRASERO_JOB_ACTION_SIZE) {
 			/* get the number of sectors. As we added -raw sector = 2352 bytes */
-			brasero_job_set_output_size_for_current_track (BRASERO_JOB (cdrdao), s1, s1 * 2352);
+			brasero_job_set_output_size_for_current_track (BRASERO_JOB (cdrdao), s1, (gint64) s1 * 2352ULL);
 			brasero_job_finished_session (BRASERO_JOB (cdrdao));
 		}
 	}
@@ -154,11 +152,24 @@ brasero_cdrdao_read_stderr_record (BraseroCdrdao *cdrdao, const gchar *line)
 		brasero_job_set_dangerous (BRASERO_JOB (cdrdao), TRUE);
 	}
 	else {
-		gchar *cuepath, *name;
-		BraseroTrack *track;
+		gchar *name = NULL;
+		gchar *cuepath = NULL;
+		BraseroTrack *track = NULL;
+		BraseroJobAction action;
+
+		/* Try to catch error could not find cue file */
+
+		/* Track could be NULL here if we're simply blanking a medium */
+		brasero_job_get_action (BRASERO_JOB (cdrdao), &action);
+		if (action == BRASERO_JOB_ACTION_ERASE)
+			return TRUE;
 
 		brasero_job_get_current_track (BRASERO_JOB (cdrdao), &track);
+		if (!track)
+			return FALSE;
+
 		cuepath = brasero_track_get_toc_source (track, FALSE);
+
 		if (!cuepath)
 			return FALSE;
 
@@ -231,7 +242,14 @@ brasero_cdrdao_set_argv_device (BraseroCdrdao *cdrdao,
 	gchar *device = NULL;
 
 	g_ptr_array_add (argv, g_strdup ("--device"));
+
+#ifdef HAVE_CAM_LIB_H
+	/* FreeBSD like that better */
+	brasero_job_get_bus_target_lun (BRASERO_JOB (cdrdao), &device);
+#else
 	brasero_job_get_device (BRASERO_JOB (cdrdao), &device);
+#endif
+
 	g_ptr_array_add (argv, device);
 }
 
@@ -243,13 +261,16 @@ brasero_cdrdao_set_argv_common_rec (BraseroCdrdao *cdrdao,
 	gchar *speed_str;
 	guint speed;
 
+	brasero_job_get_flags (BRASERO_JOB (cdrdao), &flags);
+	if (flags & BRASERO_BURN_FLAG_DUMMY)
+		g_ptr_array_add (argv, g_strdup ("--simulate"));
+
 	g_ptr_array_add (argv, g_strdup ("--speed"));
 
 	brasero_job_get_speed (BRASERO_JOB (cdrdao), &speed);
 	speed_str = g_strdup_printf ("%d", speed);
 	g_ptr_array_add (argv, speed_str);
 
-	brasero_job_get_flags (BRASERO_JOB (cdrdao), &flags);
 	if (flags & BRASERO_BURN_FLAG_OVERBURN)
 		g_ptr_array_add (argv, g_strdup ("--overburn"));
 	if (flags & BRASERO_BURN_FLAG_MULTI)
@@ -263,8 +284,6 @@ brasero_cdrdao_set_argv_common (BraseroCdrdao *cdrdao,
 	BraseroBurnFlag flags;
 
 	brasero_job_get_flags (BRASERO_JOB (cdrdao), &flags);
-	if (flags & BRASERO_BURN_FLAG_DUMMY)
-		g_ptr_array_add (argv, g_strdup ("--simulate"));
 
 	/* cdrdao manual says it is a similar option to gracetime */
 	if (flags & BRASERO_BURN_FLAG_NOGRACE)
@@ -284,7 +303,7 @@ brasero_cdrdao_set_argv_record (BraseroCdrdao *cdrdao,
 
 	brasero_job_get_input_type (BRASERO_JOB (cdrdao), &type);
         if (type.type == BRASERO_TRACK_TYPE_DISC) {
-		NautilusBurnDrive *drive;
+		BraseroDrive *drive;
 		BraseroBurnFlag flags;
 		BraseroTrack *track;
 
@@ -301,7 +320,14 @@ brasero_cdrdao_set_argv_record (BraseroCdrdao *cdrdao,
 
 		brasero_job_get_current_track (BRASERO_JOB (cdrdao), &track);
 		drive = brasero_track_get_drive_source (track);
-		g_ptr_array_add (argv, g_strdup (NCB_DRIVE_GET_DEVICE (drive)));
+
+#ifdef HAVE_CAM_LIB_H
+	/* FreeBSD like that better */
+		g_ptr_array_add (argv, brasero_drive_get_bus_target_lun_string (drive));
+#else
+		g_ptr_array_add (argv, g_strdup (brasero_drive_get_device (drive)));
+#endif
+
 	}
 	else if (type.type == BRASERO_TRACK_TYPE_IMAGE) {
 		gchar *cuepath;
@@ -350,11 +376,12 @@ brasero_cdrdao_set_argv_blank (BraseroCdrdao *cdrdao,
 	brasero_cdrdao_set_argv_device (cdrdao, argv);
 	brasero_cdrdao_set_argv_common (cdrdao, argv);
 
+	g_ptr_array_add (argv, g_strdup ("--blank-mode"));
 	brasero_job_get_flags (BRASERO_JOB (cdrdao), &flags);
-	if (!(flags & BRASERO_BURN_FLAG_FAST_BLANK)) {
-		g_ptr_array_add (argv, g_strdup ("--blank-mode"));
+	if (!(flags & BRASERO_BURN_FLAG_FAST_BLANK))
 		g_ptr_array_add (argv, g_strdup ("full"));
-	}
+	else
+		g_ptr_array_add (argv, g_strdup ("minimal"));
 
 	brasero_job_set_current_action (BRASERO_JOB (cdrdao),
 					BRASERO_BURN_ACTION_BLANKING,
@@ -371,7 +398,7 @@ brasero_cdrdao_set_argv_image (BraseroCdrdao *cdrdao,
 {
 	gchar *image = NULL, *toc = NULL;
 	BraseroBurnResult result;
-	NautilusBurnDrive *drive;
+	BraseroDrive *drive;
 	BraseroJobAction action;
 	BraseroTrackType output;
 	BraseroTrack *track;
@@ -382,14 +409,19 @@ brasero_cdrdao_set_argv_image (BraseroCdrdao *cdrdao,
 
 	brasero_job_get_current_track (BRASERO_JOB (cdrdao), &track);
 	drive = brasero_track_get_drive_source (track);
-	g_ptr_array_add (argv, g_strdup (NCB_DRIVE_GET_DEVICE (drive)));
+
+#ifdef HAVE_CAM_LIB_H
+	/* FreeBSD like that better */
+	g_ptr_array_add (argv, brasero_drive_get_bus_target_lun_string (drive));
+#else
+	g_ptr_array_add (argv, g_strdup (brasero_drive_get_device (drive)));
+#endif
 
 	g_ptr_array_add (argv, g_strdup ("--read-raw"));
 
 	/* This is done so that if a cue file is required we first generate
 	 * a temporary toc file that will be later converted to a cue file.
 	 * The datafile is written where it should be from the start. */
-
 	brasero_job_get_output_type (BRASERO_JOB (cdrdao), &output);
 	if (output.subtype.img_format == BRASERO_IMAGE_FORMAT_CDRDAO) {
 		result = brasero_job_get_image_output (BRASERO_JOB (cdrdao),
@@ -477,7 +509,7 @@ brasero_cdrdao_set_argv (BraseroProcess *process,
 
 			brasero_job_set_output_size_for_current_track (BRASERO_JOB (cdrdao),
 								       sectors,
-								       sectors * 2352);
+								       sectors * 2352ULL);
 		}
 		else
 			return BRASERO_BURN_NOT_SUPPORTED;
@@ -577,9 +609,12 @@ brasero_cdrdao_export_caps (BraseroPlugin *plugin, gchar **error)
 	g_slist_free (input);
 
 	/* cdrdao is used to burn images so it can't APPEND and the disc must
-	 * have been blanked before (it can't overwrite) */
+	 * have been blanked before (it can't overwrite)
+	 * NOTE: BRASERO_MEDIUM_FILE is needed here because of restriction API
+	 * when we output an image. */
 	brasero_plugin_set_flags (plugin,
-				  media_w,
+				  media_w|
+				  BRASERO_MEDIUM_FILE,
 				  BRASERO_BURN_FLAG_DAO|
 				  BRASERO_BURN_FLAG_BURNPROOF|
 				  BRASERO_BURN_FLAG_OVERBURN|
