@@ -7,19 +7,21 @@
  ***************************************************************************/
 
 /*
- *  This program is free software; you can redistribute it and/or modify
+ *  Brasero is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Brasero is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -102,15 +104,17 @@ brasero_toc2cue_read_stderr (BraseroProcess *process,
 	
 	output = fopen (toc_path, "w");
 	if (!output) {
+                int errsv = errno;
+
 		g_io_channel_unref (source);
 
 		g_free (img_path);
 		g_free (toc_path);
 
 		brasero_job_error (BRASERO_JOB (process),
-				   g_error_new (BRASERO_BURN_ERROR,
-						BRASERO_BURN_ERROR_GENERAL,
-						strerror (errno)));
+				   g_error_new_literal (BRASERO_BURN_ERROR,
+							BRASERO_BURN_ERROR_GENERAL,
+							g_strerror (errsv)));
 		return BRASERO_BURN_OK;
 	}
 
@@ -137,6 +141,8 @@ brasero_toc2cue_read_stderr (BraseroProcess *process,
 		}
 
 		if (!fwrite (buffer, strlen (buffer), 1, output)) {
+                        int errsv = errno;
+
 			g_free (buffer);
 
 			fclose (output);
@@ -148,9 +154,9 @@ brasero_toc2cue_read_stderr (BraseroProcess *process,
 			g_free (toc_path);
 
 			brasero_job_error (BRASERO_JOB (process),
-					   g_error_new (BRASERO_BURN_ERROR,
-							BRASERO_BURN_ERROR_GENERAL,
-							strerror (errno)));
+					   g_error_new_literal (BRASERO_BURN_ERROR,
+								BRASERO_BURN_ERROR_GENERAL,
+								g_strerror (errsv)));
 			return BRASERO_BURN_OK;
 		}
 
@@ -172,18 +178,22 @@ brasero_toc2cue_read_stderr (BraseroProcess *process,
 	/* the previous track image path will now be a link pointing to the
 	 * image path of the new track just created */
 	if (g_rename (tmp_img_path, img_path)) {
+                int errsv = errno;
+
 		brasero_job_error (BRASERO_JOB (self),
-				   g_error_new (BRASERO_BURN_ERROR,
-						BRASERO_BURN_ERROR_GENERAL,
-						strerror (errno)));
+				   g_error_new_literal (BRASERO_BURN_ERROR,
+							BRASERO_BURN_ERROR_GENERAL,
+							g_strerror (errsv)));
 		return BRASERO_BURN_OK;
 	}
 
 	if (link (img_path, tmp_img_path)) {
+                int errsv = errno;
+
 		brasero_job_error (BRASERO_JOB (self),
-				   g_error_new (BRASERO_BURN_ERROR,
-						BRASERO_BURN_ERROR_GENERAL,
-						strerror (errno)));
+				   g_error_new_literal (BRASERO_BURN_ERROR,
+							BRASERO_BURN_ERROR_GENERAL,
+							g_strerror (errsv)));
 		return BRASERO_BURN_OK;
 	} /* symlink () could also be used */
 
@@ -263,7 +273,7 @@ brasero_toc2cue_post (BraseroJob *job)
 		priv->output = NULL;
 	}
 
-	return BRASERO_BURN_OK;
+	return brasero_job_finished_session (job);
 }
 
 static void
