@@ -7,28 +7,35 @@
  ***************************************************************************/
 
 /*
- *  This program is free software; you can redistribute it and/or modify
+ *  Brasero is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Brasero is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
+/* This is for getline() and isblank() */
+#define _GNU_SOURCE
+
 #include <string.h>
 #include <stdlib.h>
 #include <errno.h>
+#include <stdio.h>
+#include <ctype.h>
 
 #include <glib.h>
 #include <glib-object.h>
@@ -93,7 +100,9 @@ brasero_local_track_get_download_size (BraseroLocalTrack *self,
 	priv = BRASERO_LOCAL_TRACK_PRIVATE (self);
 
 	enumerator = g_file_enumerate_children (src,
-						G_FILE_ATTRIBUTE_STANDARD_TYPE,
+						G_FILE_ATTRIBUTE_STANDARD_TYPE ","
+						G_FILE_ATTRIBUTE_STANDARD_NAME ","
+						G_FILE_ATTRIBUTE_STANDARD_SIZE,
 						G_FILE_QUERY_INFO_NONE,	/* follow symlinks */
 						priv->cancel,
 						error);
@@ -130,6 +139,9 @@ brasero_local_track_progress_cb (goffset current_num_bytes,
 
 	priv = BRASERO_LOCAL_TRACK_PRIVATE (self);
 
+	if (!priv->data_size)
+		return;
+
 	brasero_job_start_progress (BRASERO_JOB (self), FALSE);
 	brasero_job_set_progress (BRASERO_JOB (self),
 				  (gdouble) (priv->read_bytes + current_num_bytes) /
@@ -149,6 +161,7 @@ brasero_local_track_file_transfer (BraseroLocalTrack *self,
 	priv = BRASERO_LOCAL_TRACK_PRIVATE (self);
 
 	name = g_file_get_basename (src);
+	BRASERO_JOB_LOG (self, "Downloading %s", name);
 	string = g_strdup_printf (_("Copying `%s` locally"), name);
 	g_free (name);
 
@@ -182,8 +195,11 @@ brasero_local_track_recursive_transfer (BraseroLocalTrack *self,
 
 	priv = BRASERO_LOCAL_TRACK_PRIVATE (self);
 
+	BRASERO_JOB_LOG (self, "Downloading directory contents");
 	enumerator = g_file_enumerate_children (src,
-						G_FILE_ATTRIBUTE_STANDARD_TYPE,
+						G_FILE_ATTRIBUTE_STANDARD_TYPE ","
+						G_FILE_ATTRIBUTE_STANDARD_NAME ","
+						G_FILE_ATTRIBUTE_STANDARD_SIZE,
 						G_FILE_QUERY_INFO_NONE,	/* follow symlinks */
 						priv->cancel,
 						error);
@@ -195,13 +211,6 @@ brasero_local_track_recursive_transfer (BraseroLocalTrack *self,
 		GFile *dest_child;
 		GFile *src_child;
 
-		if (error) {
-			g_file_enumerator_close (enumerator, priv->cancel, NULL);
-			g_object_unref (enumerator);
-			g_object_unref (info);
-			return BRASERO_BURN_ERR;
-		}
-
 		src_child = g_file_get_child (src, g_file_info_get_name (info));
 		dest_child = g_file_get_child (dest, g_file_info_get_name (info));
 
@@ -209,14 +218,16 @@ brasero_local_track_recursive_transfer (BraseroLocalTrack *self,
 			gchar *path;
 
 			path = g_file_get_path (dest_child);
+			BRASERO_JOB_LOG (self, "Creating directory %s", path);
 
 			/* create a directory with the same name and explore it */
-			if (g_mkdir (path, 700)) {
+			if (g_mkdir (path, S_IRWXU)) {
+                                int errsv = errno;
 				g_set_error (error,
 					     BRASERO_BURN_ERROR,
 					     BRASERO_BURN_ERROR_GENERAL,
 					     _("a directory couldn't be created (%s)"),
-					     strerror (errno));
+					     g_strerror (errsv));
 				result = BRASERO_BURN_ERR;
 			}
 			else {
@@ -244,14 +255,13 @@ brasero_local_track_recursive_transfer (BraseroLocalTrack *self,
 			g_file_enumerator_close (enumerator, priv->cancel, NULL);
 			g_object_unref (enumerator);
 			return BRASERO_BURN_ERR;
-
 		}
 	}
 
 	g_file_enumerator_close (enumerator, priv->cancel, NULL);
 	g_object_unref (enumerator);
 
-	return BRASERO_BURN_OK;
+	return ((*error) == NULL)? BRASERO_BURN_OK:BRASERO_BURN_ERR;
 }
 
 static BraseroBurnResult
@@ -262,22 +272,28 @@ brasero_local_track_transfer (BraseroLocalTrack *self,
 {
 	GFileInfo *info;
 	BraseroBurnResult result;
+	GError *local_error = NULL;
 	BraseroLocalTrackPrivate *priv;
 
 	priv = BRASERO_LOCAL_TRACK_PRIVATE (self);
 
 	/* Retrieve some information about the file we have to copy */
 	info = g_file_query_info (src,
-				  G_FILE_ATTRIBUTE_STANDARD_TYPE,
+				  G_FILE_ATTRIBUTE_STANDARD_TYPE ","
+				  G_FILE_ATTRIBUTE_STANDARD_SIZE,
 				  G_FILE_QUERY_INFO_NONE, /* follow symlinks */
 				  priv->cancel,
-				  error);
-	if (!info || error)
+				  &local_error);
+	if (!info || local_error) {
+		g_propagate_error (error, local_error);
 		return BRASERO_BURN_ERR;
+	}
 
 	/* Retrieve the size of all the data. */
-	if (g_file_info_get_file_type (info) != G_FILE_TYPE_DIRECTORY)
+	if (g_file_info_get_file_type (info) != G_FILE_TYPE_DIRECTORY) {
+		BRASERO_JOB_LOG (self, "Downloading file (size = %lli)", g_file_info_get_size (info));
 		priv->data_size = g_file_info_get_size (info);
+	}
 	else
 		brasero_local_track_get_download_size (self, src, error);
 
@@ -288,7 +304,12 @@ brasero_local_track_transfer (BraseroLocalTrack *self,
 		gchar *dest_path;
 
 		dest_path = g_file_get_path (dest);
-		if (g_mkdir_with_parents (dest_path, 700)) {
+
+		/* remove the temporary file that was created */
+		g_remove (dest_path);
+		if (g_mkdir_with_parents (dest_path, S_IRWXU)) {
+                        int errsv = errno;
+
 			g_free (dest_path);
 			g_object_unref (info);
 
@@ -296,14 +317,17 @@ brasero_local_track_transfer (BraseroLocalTrack *self,
 				     BRASERO_BURN_ERROR,
 				     BRASERO_BURN_ERROR_GENERAL,
 				     _("a directory couldn't be created (%s)"),
-				     strerror (errno));
+				     g_strerror (errsv));
 			return BRASERO_BURN_ERR;
 		}
 
+		BRASERO_JOB_LOG (self, "Created directory %s", dest_path);
 		g_free (dest_path);
+
 		result = brasero_local_track_recursive_transfer (self, src, dest, error);
 	}
 	else {
+		g_file_delete (dest, priv->cancel, NULL);
 		result = brasero_local_track_file_transfer (self, src, dest, error);
 		priv->read_bytes += g_file_info_get_size (info);
 	}
@@ -319,7 +343,7 @@ brasero_local_track_transfer (BraseroLocalTrack *self,
 
 static gchar *
 brasero_local_track_translate_uri (BraseroLocalTrack *self,
-				   gchar *uri)
+				   const gchar *uri)
 {
 	gchar *newuri;
 	gchar *parent;
@@ -327,18 +351,16 @@ brasero_local_track_translate_uri (BraseroLocalTrack *self,
 
 	priv = BRASERO_LOCAL_TRACK_PRIVATE (self);
 
-	if (uri == NULL)
+	if (!uri)
 		return NULL;
 
 	/* see if it is a local file */
 	if (g_str_has_prefix (uri, "file://"))
-		return uri;
+		return g_strdup (uri);
 
 	/* see if it was downloaded itself */
 	newuri = g_hash_table_lookup (priv->nonlocals, uri);
 	if (newuri) {
-		g_free (uri);
-
 		/* we copy this string as it will be freed when freeing 
 		 * downloaded GSList */
 		return g_strdup (newuri);
@@ -351,11 +373,11 @@ brasero_local_track_translate_uri (BraseroLocalTrack *self,
 
 		tmp = g_hash_table_lookup (priv->nonlocals, parent);
 		if (tmp) {
-			newuri = g_strconcat (tmp,
-					      uri + strlen (parent),
-					      NULL);
+			newuri = g_build_path (G_DIR_SEPARATOR_S,
+					       tmp,
+					       uri + strlen (parent),
+					       NULL);
 			g_free (parent);
-			g_free (uri);
 			return newuri;
 		}
 
@@ -365,44 +387,70 @@ brasero_local_track_translate_uri (BraseroLocalTrack *self,
 	}
 
 	/* that should not happen */
-	g_warning ("Can't find a downloaded parent for this non local uri.\n");
+	BRASERO_JOB_LOG (self, "Can't find a downloaded parent for %s", uri);
 
 	g_free (parent);
-	g_free (uri);
 	return NULL;
 }
 
 static BraseroBurnResult
 brasero_local_track_read_checksum (BraseroLocalTrack *self)
 {
-	gint bytes;
+	gint size;
 	FILE *file;
-	gchar *path;
-	gchar buffer [512];
+	gchar *name;
+	gchar *source;
+	gchar *line = NULL;
+	size_t line_len = 0;
+	BraseroTrack *track = NULL;
 	BraseroLocalTrackPrivate *priv;
 
 	priv = BRASERO_LOCAL_TRACK_PRIVATE (self);
 
-	/* get the file_checksum from the md5 file */
-	path = g_filename_from_uri (priv->checksum_path, NULL, NULL);
-	file = fopen (path, "r");
-	g_free (path);
+	/* get the file_checksum from the checksum file */
+	file = fopen (priv->checksum_path, "r");
 
-	if (!file)
+	/* get the name of the file that was downloaded */
+	brasero_job_get_current_track (BRASERO_JOB (self), &track);
+	source = brasero_track_get_image_source (track, TRUE);
+	name = g_path_get_basename (source);
+	g_free (source);
+
+	if (!file) {
+		g_free (name);
+		BRASERO_JOB_LOG (self, "Impossible to open the downloaded checksum file");
 		return BRASERO_BURN_ERR;
+	}
 
-	bytes = fread (buffer,
-		       1,
-		       g_checksum_type_get_length (priv->gchecksum_type) - 1,
-		       file);
+	/* find a line with the name of our file (there could be several ones) */
+	BRASERO_JOB_LOG (self, "Searching %s in file", name);
+	while ((size = getline (&line, &line_len, file)) > 0) {
+		if (strstr (line, name)) {
+			gchar *ptr;
+
+			/* Skip blanks */
+			ptr = line;
+			while (isblank (*ptr)) { ptr ++; size --; }
+
+			if (g_checksum_type_get_length (priv->checksum_type) * 2 > size) {
+				g_free (line);
+				line = NULL;
+				continue;
+			}
+	
+			ptr [g_checksum_type_get_length (priv->gchecksum_type) * 2] = '\0';
+			priv->checksum = g_strdup (ptr);
+			g_free (line);
+			BRASERO_JOB_LOG (self, "Found checksum %s", priv->checksum);
+			break;
+		}
+
+		g_free (line);
+		line = NULL;
+	}
 	fclose (file);
-	buffer [g_checksum_type_get_length (priv->gchecksum_type)] = '\0';
 
-	if (bytes != g_checksum_type_get_length (priv->gchecksum_type) - 1)
-		return BRASERO_BURN_ERR;
-
-	priv->checksum = g_strdup (buffer);
-	return BRASERO_BURN_OK;
+	return (priv->checksum? BRASERO_BURN_OK:BRASERO_BURN_ERR);
 }
 
 static BraseroBurnResult
@@ -413,10 +461,13 @@ brasero_local_track_download_checksum (BraseroLocalTrack *self)
 	BraseroTrack *track;
 	gchar *checksum_src;
 	GFile *src, *dest;
+	GFile *tmp;
 	gchar *uri;
 
 	priv = BRASERO_LOCAL_TRACK_PRIVATE (self);
 
+	BRASERO_JOB_LOG (self, "Copying checksum file");
+
 	/* generate a unique name for destination */
 	result = brasero_job_get_tmp_file (BRASERO_JOB (self),
 					   NULL,
@@ -431,10 +482,11 @@ brasero_local_track_download_checksum (BraseroLocalTrack *self)
 					TRUE);
 
 	/* This is an image. See if there is any checksum sitting in the same
-	 * directory to check our download integrity. Try all types of checksum. */
+	 * directory to check our download integrity.
+	 * Try all types of checksum. */
 	brasero_job_get_current_track (BRASERO_JOB (self), &track);
 	uri = brasero_track_get_image_source (track, TRUE);
-	dest = g_file_new_for_uri (priv->checksum_path);
+	dest = g_file_new_for_path (priv->checksum_path);
 
 	/* Try with three difference sources */
 	checksum_src = g_strdup_printf ("%s.md5", uri);
@@ -479,14 +531,36 @@ brasero_local_track_download_checksum (BraseroLocalTrack *self)
 					       NULL);
 	g_object_unref (src);
 
+	if (result == BRASERO_BURN_OK) {
+		priv->gchecksum_type = G_CHECKSUM_SHA256;
+		priv->checksum_type = BRASERO_CHECKSUM_SHA256;
+		goto end;
+	}
+
+	/* There are also ftp sites that includes all images checksum in one big
+	 * SHA1 file. */
+	tmp = g_file_new_for_uri (uri);
+	src = g_file_get_parent (tmp);
+	g_object_unref (tmp);
+
+	tmp = src;
+	src = g_file_get_child_for_display_name (tmp, "SHA1SUM", NULL);
+	g_object_unref (tmp);
+
+	result = brasero_local_track_transfer (self,
+					       src,
+					       dest,
+					       NULL);
+	g_object_unref (src);
+
 	if (result != BRASERO_BURN_OK) {
 		g_free (uri);
 		g_object_unref (dest);
 		goto error;
 	}
 
-	priv->gchecksum_type = G_CHECKSUM_SHA256;
-	priv->checksum_type = BRASERO_CHECKSUM_SHA256;
+	priv->gchecksum_type = G_CHECKSUM_SHA1;
+	priv->checksum_type = BRASERO_CHECKSUM_SHA1;
 
 end:
 
@@ -496,14 +570,16 @@ end:
 	return result;
 
 error:
+
 	/* we give up */
+	BRASERO_JOB_LOG (self, "No checksum file available");
 	g_free (priv->checksum_path);
 	priv->checksum_path = NULL;
 	return result;
 }
 
-static gboolean
-brasero_local_track_thread_finished (BraseroLocalTrack *self)
+static BraseroBurnResult
+brasero_local_track_update_track (BraseroLocalTrack *self)
 {
 	BraseroTrack *track;
 	BraseroTrackType input;
@@ -511,6 +587,110 @@ brasero_local_track_thread_finished (BraseroLocalTrack *self)
 
 	priv = BRASERO_LOCAL_TRACK_PRIVATE (self);
 
+	/* now we update all the track with the local uris in retval */
+	brasero_job_get_current_track (BRASERO_JOB (self), &track);
+	track = brasero_track_copy (track);
+	brasero_track_get_type (track, &input);
+
+	/* make a copy of the tracks instead of modifying them */
+	switch (input.type) {
+	case BRASERO_TRACK_TYPE_DATA: {
+		GSList *next;
+		GSList *grafts;
+		GSList *unreadable;
+
+		grafts = brasero_track_get_data_grafts_source (track);
+		for (; grafts; grafts = grafts->next) {
+			BraseroGraftPt *graft;
+			gchar *uri;
+
+			graft = grafts->data;
+			uri = brasero_local_track_translate_uri (self, graft->uri);
+			if (uri) {
+				g_free (graft->uri);
+				graft->uri = uri;
+			}
+		}
+
+		BRASERO_JOB_LOG (self, "Translating unreadable");
+
+		/* Translate the globally excluded.
+		 * NOTE: if we can't find a parent for an excluded URI that
+		 * means it shouldn't be included. */
+		unreadable = brasero_track_get_data_excluded_source (track, FALSE);
+		for (; unreadable; unreadable = next) {
+			gchar *new_uri;
+
+			next = unreadable->next;
+			new_uri = brasero_local_track_translate_uri (self, unreadable->data);
+			g_free (unreadable->data);
+
+			if (new_uri)
+				unreadable->data = new_uri;
+			else
+				unreadable = g_slist_remove (unreadable, unreadable->data);
+		}
+	}
+	break;
+
+	case BRASERO_TRACK_TYPE_AUDIO: {
+		gchar *uri;
+		gchar *newuri;
+
+		uri = brasero_track_get_audio_source (track, TRUE);
+		newuri = brasero_local_track_translate_uri (self, uri);
+		brasero_track_set_audio_source (track,
+						newuri,
+						input.subtype.audio_format);
+		g_free (uri);
+	}
+	break;
+
+	case BRASERO_TRACK_TYPE_IMAGE: {
+		gchar *uri;
+		gchar *newtoc;
+		gchar *newimage;
+
+		uri = brasero_track_get_image_source (track, TRUE);
+		newimage = brasero_local_track_translate_uri (self, uri);
+		g_free (uri);
+
+		uri = brasero_track_get_toc_source (track, TRUE);
+		newtoc = brasero_local_track_translate_uri (self, uri);
+		g_free (uri);
+
+		brasero_track_set_image_source (track,
+						newimage,
+						newtoc,
+						input.subtype.img_format);
+	}
+	break;
+
+	default:
+		BRASERO_JOB_NOT_SUPPORTED (self);
+	}
+
+	if (priv->download_checksum)
+		brasero_track_set_checksum (track,
+					    priv->checksum_type,
+					    priv->checksum);
+
+	brasero_job_add_track (BRASERO_JOB (self), track);
+
+	/* It's good practice to unref the track afterwards as we don't need it
+	 * anymore. BraseroTaskCtx refs it. */
+	brasero_track_unref (track);
+
+	return BRASERO_BURN_OK;
+}
+
+static gboolean
+brasero_local_track_thread_finished (BraseroLocalTrack *self)
+{
+	BraseroLocalTrackPrivate *priv;
+
+	priv = BRASERO_LOCAL_TRACK_PRIVATE (self);
+
 	priv->thread_id = 0;
 
 	if (priv->cancel) {
@@ -529,68 +709,8 @@ brasero_local_track_thread_finished (BraseroLocalTrack *self)
 		return FALSE;
 	}
 
-	/* now we update all the track with the local uris in retval */
-	brasero_job_get_current_track (BRASERO_JOB (self), &track);
-	track = brasero_track_copy (track);
-	brasero_track_get_type (track, &input);
+	brasero_local_track_update_track (self);
 
-	/* make a copy of the tracks instead of modifying them */
-	switch (input.type) {
-	case BRASERO_TRACK_TYPE_DATA: {
-		GSList *grafts;
-		GSList *unreadable;
-
-		grafts = brasero_track_get_data_grafts_source (track);
-		for (; grafts; grafts = grafts->next) {
-			BraseroGraftPt *graft;
-
-			graft = grafts->data;
-			graft->uri = brasero_local_track_translate_uri (self, graft->uri);
-		}
-
-		/* translate the globally excluded */
-		unreadable = brasero_track_get_data_excluded_source (track, FALSE);
-		for (; unreadable; unreadable = unreadable->next)
-			unreadable->data = brasero_local_track_translate_uri (self, unreadable->data);
-	}
-	break;
-
-	case BRASERO_TRACK_TYPE_AUDIO: {
-		gchar *uri;
-
-		uri = brasero_track_get_audio_source (track, TRUE);
-		uri = brasero_local_track_translate_uri (self, uri);
-		brasero_track_set_audio_source (track, uri, input.subtype.audio_format); 
-	}
-	break;
-
-	case BRASERO_TRACK_TYPE_IMAGE: {
-		gchar *uri;
-
-		uri = brasero_track_get_image_source (track, TRUE);
-		uri = brasero_local_track_translate_uri (self, uri);
-		brasero_track_set_image_source (track, uri, NULL, input.subtype.img_format);
-
-		uri = brasero_track_get_toc_source (track, TRUE);
-		uri = brasero_local_track_translate_uri (self, uri);
-		brasero_track_set_image_source (track, NULL, uri, input.subtype.img_format);
-	}
-	break;
-
-	default:
-		BRASERO_JOB_NOT_SUPPORTED (self);
-	}
-
-	if (priv->download_checksum)
-		brasero_track_set_checksum (track,
-					    priv->checksum_type,
-					    priv->checksum);
-
-	brasero_job_add_track (BRASERO_JOB (self), track);
-
-	/* It's good practice to unref the track afterwards as we don't need it
-	 * anymore. BraseroTaskCtx refs it. */
-	brasero_track_unref (track);
 	brasero_job_finished_track (BRASERO_JOB (self));
 	return FALSE;
 }
@@ -673,6 +793,7 @@ _foreach_non_local_cb (const gchar *uri,
 		       const gchar *localuri,
 		       gpointer *data)
 {
+	BraseroLocalTrack *self = BRASERO_LOCAL_TRACK (data);
 	BraseroLocalTrackPrivate *priv;
 	GFile *file, *tmpfile;
 	gchar *parent;
@@ -683,8 +804,11 @@ _foreach_non_local_cb (const gchar *uri,
 	/* check that is hasn't any parent in the hash */
 	parent = g_path_get_dirname (uri);
 	while (parent [1] != '\0') {
-		localuri = g_hash_table_lookup (priv->nonlocals, parent);
-		if (localuri) {
+		gchar *uri_local;
+
+		uri_local = g_hash_table_lookup (priv->nonlocals, parent);
+		if (uri_local) {
+			BRASERO_JOB_LOG (self, "Parent for %s was found %s", uri, parent);
 			g_free (parent);
 			return TRUE;
 		}
@@ -701,6 +825,7 @@ _foreach_non_local_cb (const gchar *uri,
 	tmpfile = g_file_new_for_uri (localuri);
 	priv->dest_list = g_slist_append (priv->dest_list, tmpfile);
 
+	BRASERO_JOB_LOG (self, "%s set to be downloaded to %s", uri, localuri);
 	return FALSE;
 }
 
@@ -715,7 +840,11 @@ brasero_local_track_add_if_non_local (BraseroLocalTrack *self,
 
 	priv = BRASERO_LOCAL_TRACK_PRIVATE (self);
 
-	if (!uri || g_str_has_prefix (uri, "file://"))
+	if (!uri
+	||   uri [0] == '\0'
+	||   uri [0] == '/'
+	||   g_str_has_prefix (uri, "file://")
+	||   g_str_has_prefix (uri, "burn://"))
 		return BRASERO_BURN_OK;
 
 	/* add it to the list or uris to download */
@@ -725,32 +854,6 @@ brasero_local_track_add_if_non_local (BraseroLocalTrack *self,
 							 NULL,
 							 g_free);
 
-	if (g_str_has_prefix (uri, "burn://")) {
-		GFile *file;
-		GFileInfo *info;
-
-		/* this is a special case for burn:// uris */
-		file = g_file_new_for_uri (uri);
-		info = g_file_query_info (file, "burn::backing-file", 0, NULL, error);
-		g_object_unref (file);
-
-		if (!info)
-			return BRASERO_BURN_ERR;
-
-		localuri = g_strdup (g_file_info_get_attribute_byte_string (info, "burn::backing-file"));
-		g_object_unref (info);
-		if (!localuri) {
-			g_set_error (error,
-				     BRASERO_BURN_ERROR,
-				     BRASERO_BURN_ERROR_GENERAL,
-				     _("impossible to retrieve local file path"));
-			return BRASERO_BURN_ERR;
-		}
-
-		g_hash_table_insert (priv->nonlocals, (gpointer) uri, localuri);
-		return BRASERO_BURN_OK;
-	}
-
 	/* generate a unique name */
 	result = brasero_job_get_tmp_file (BRASERO_JOB (self),
 					   NULL,
@@ -769,7 +872,7 @@ brasero_local_track_add_if_non_local (BraseroLocalTrack *self,
 
 	/* we don't want to replace it if it has already been downloaded */
 	if (!g_hash_table_lookup (priv->nonlocals, uri))
-		g_hash_table_insert (priv->nonlocals, (gpointer) uri, localuri);
+		g_hash_table_insert (priv->nonlocals, g_strdup (uri), localuri);
 
 	return BRASERO_BURN_OK;
 }
@@ -779,6 +882,7 @@ brasero_local_track_start (BraseroJob *job,
 			   GError **error)
 {
 	BraseroLocalTrackPrivate *priv;
+	BraseroBurnResult result;
 	BraseroJobAction action;
 	BraseroLocalTrack *self;
 	BraseroTrackType input;
@@ -804,6 +908,8 @@ brasero_local_track_start (BraseroJob *job,
 	brasero_job_get_current_track (job, &track);
 	brasero_job_get_input_type (job, &input);
 
+	result = BRASERO_BURN_OK;
+
 	/* make a list of all non local uris to be downloaded and put them in a
 	 * list to avoid to download the same file twice. */
 	switch (input.type) {
@@ -814,35 +920,47 @@ brasero_local_track_start (BraseroJob *job,
 			BraseroGraftPt *graft;
 
 			graft = grafts->data;
-			brasero_local_track_add_if_non_local (self, graft->uri, error);
+			result = brasero_local_track_add_if_non_local (self, graft->uri, error);
+			if (result != BRASERO_BURN_OK)
+				break;
 		}
+
 		break;
 
 	case BRASERO_TRACK_TYPE_AUDIO:
+		/* NOTE: don't delete URI as they will be inserted in hash */
 		uri = brasero_track_get_audio_source (track, TRUE);
-		brasero_local_track_add_if_non_local (self, uri, error);
+		result = brasero_local_track_add_if_non_local (self, uri, error);
 		g_free (uri);
 		break;
 
 	case BRASERO_TRACK_TYPE_IMAGE:
+		/* NOTE: don't delete URI as they will be inserted in hash */
 		uri = brasero_track_get_image_source (track, TRUE);
-		brasero_local_track_add_if_non_local (self, uri, error);
+		result = brasero_local_track_add_if_non_local (self, uri, error);
 		g_free (uri);
 
+		if (result != BRASERO_BURN_OK)
+			break;
+
 		priv->download_checksum = TRUE;
 
 		uri = brasero_track_get_toc_source (track, TRUE);
-		brasero_local_track_add_if_non_local (self, uri, error);
+		result = brasero_local_track_add_if_non_local (self, uri, error);
 		g_free (uri);
+
 		break;
 
 	default:
 		BRASERO_JOB_NOT_SUPPORTED (self);
 	}
 
+	if (result != BRASERO_BURN_OK)
+		return result;
+
 	/* see if there is anything to download */
 	if (!priv->nonlocals) {
-		BRASERO_JOB_LOG (self, "no foreign URIs");
+		BRASERO_JOB_LOG (self, "no remote URIs");
 		return BRASERO_BURN_NOT_RUNNING;
 	}
 
@@ -853,13 +971,6 @@ brasero_local_track_start (BraseroJob *job,
 				     (GHRFunc) _foreach_non_local_cb,
 				     job);
 
-	/* if there are files in list then download them otherwise stop */
-	if (!priv->src_list) {
-		/* that means there were only burn:// uris in nonlocals */
-		BRASERO_JOB_LOG (self, "no foreign URIs");
-		return BRASERO_BURN_NOT_RUNNING;
-	}
-
 	return brasero_local_track_start_thread (self, error);
 }
 
@@ -896,11 +1007,13 @@ brasero_local_track_stop (BraseroJob *job,
 	if (priv->src_list) {
 		g_slist_foreach (priv->src_list, (GFunc) g_object_unref, NULL);
 		g_slist_free (priv->src_list);
+		priv->src_list = NULL;
 	}
 
 	if (priv->dest_list) {
 		g_slist_foreach (priv->dest_list, (GFunc) g_object_unref, NULL);
 		g_slist_free (priv->dest_list);
+		priv->dest_list = NULL;
 	}
 
 	if (priv->nonlocals) {
@@ -965,7 +1078,14 @@ brasero_local_track_export_caps (BraseroPlugin *plugin, gchar **error)
 	caps = brasero_caps_audio_new (BRASERO_PLUGIN_IO_ACCEPT_FILE,
 				       BRASERO_AUDIO_FORMAT_UNDEFINED|
 				       BRASERO_AUDIO_FORMAT_4_CHANNEL|
-				       BRASERO_AUDIO_FORMAT_RAW);
+				       BRASERO_AUDIO_FORMAT_RAW|
+				       BRASERO_VIDEO_FORMAT_UNDEFINED|
+				       BRASERO_VIDEO_FORMAT_VCD|
+				       BRASERO_VIDEO_FORMAT_VIDEO_DVD|
+				       BRASERO_AUDIO_FORMAT_AC3|
+				       BRASERO_AUDIO_FORMAT_MP2|
+				       BRASERO_AUDIO_FORMAT_44100|
+				       BRASERO_AUDIO_FORMAT_48000);
 	brasero_plugin_process_caps (plugin, caps);
 	g_slist_free (caps);
 
@@ -973,7 +1093,7 @@ brasero_local_track_export_caps (BraseroPlugin *plugin, gchar **error)
 	brasero_plugin_process_caps (plugin, caps);
 	g_slist_free (caps);
 
-	brasero_plugin_set_process_flags (plugin, BRASERO_PLUGIN_RUN_FIRST);
+	brasero_plugin_set_process_flags (plugin, BRASERO_PLUGIN_RUN_PREPROCESSING);
 
 	return BRASERO_BURN_OK;
 }
